package controllers

import (
	"encoding/json"
	"fmt"
	"github.com/beego/beego/v2/server/web"
	"hongze/hz_crm_api/cache"
	"hongze/hz_crm_api/models"
	"hongze/hz_crm_api/utils"
	"net/http"
	"net/url"
)

type BaseCommonController struct {
	web.Controller
}

func (this *BaseCommonController) Prepare() {
	var requestBody string
	method := this.Ctx.Input.Method()
	if method == "GET" {
		requestBody = this.Ctx.Request.RequestURI
	} else {
		requestBody, _ = url.QueryUnescape(string(this.Ctx.Input.RequestBody))
	}
	ip := this.Ctx.Input.IP()
	apiLog.Println("请求地址：", this.Ctx.Input.URI(), "RequestBody：", requestBody, "IP：", ip)
}

func (c *BaseCommonController) ServeJSON(encoding ...bool) {
	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		go utils.SendEmail("异常提醒:"+utils.RunMode, "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值", utils.EmailSendToUsers)
		return
	}
	baseRes := c.Data["json"].(*models.BaseResponse)
	if baseRes != nil && !baseRes.Success && baseRes.IsSendEmail {
		go utils.SendEmail(utils.APPNAME+"【"+utils.RunMode+"】"+"失败提醒", "URI:"+c.Ctx.Input.URI()+" ErrMsg:"+baseRes.ErrMsg+";Msg"+baseRes.Msg, utils.EmailSendToUsers)
	}

	//新增uuid记录
	{
		urlPath := c.Ctx.Request.URL.Path
		if _, ok := AdminOperateRecordMap[urlPath]; !ok {
			var adminId int
			var realName, params, ip, uriStr string
			ip = c.Ctx.Input.IP()
			uuid := c.Ctx.Input.Header("Uuid")
			uri := c.Ctx.Input.URI()
			userAgent := c.Ctx.Input.Header("User-Agent")

			uriStr, _ = url.PathUnescape(uri)
			if uriStr != "" {
				uri = uriStr
			}

			if c.Ctx.Input.Method() == "GET" {
				paramsJson, _ := json.Marshal(c.Ctx.Input.Params())
				params = string(paramsJson)
			} else {
				params = string(c.Ctx.Input.RequestBody)
			}
			header := c.Ctx.Request.Header
			headerJson, _ := json.Marshal(header)
			headerStr := string(headerJson)
			go cache.AdminOperateRecord(adminId, realName, uuid, uri, params, ip, userAgent, headerStr)
		}
	}

	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseCommonController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	params := c.Ctx.Input.Params()
	fmt.Println("params")
	fmt.Println(params)
	requestBody, _ := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	apiLog.Println("请求地址：", c.Ctx.Input.URI(), "Authorization:", c.Ctx.Input.Header("Authorization"), "RequestBody：", requestBody, "ResponseBody", string(content), "IP：", ip)

	// 如果不是debug分支的话，那么需要加密返回
	if utils.RunMode != "debug" {
		content = utils.DesBase64Encrypt(content)
		// get请求时，不加双引号就获取不到数据，不知道什么原因，所以还是在前后加上双引号吧
		content = []byte(`"` + string(content) + `"`)
	}
	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}
	return c.Ctx.Output.Body(content)
}
