package controllers

import (
	"encoding/json"
	"hongze/hz_crm_api/models"
	"hongze/hz_crm_api/models/system"
	"hongze/hz_crm_api/utils"
	"strings"
	"time"
)

// 部门组
type SysTeamController struct {
	BaseAuthController
}

// @Title 新增分组
// @Description 新增分组接口
// @Param	request	body system.SysTeamAddReq true "type json string"
// @Success 200 新增成功
// @router /team/add [post]
func (this *SysTeamController) Add() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysTeamAddReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.GroupId <= 0 {
		br.Msg = "请选择部门"
		br.ErrMsg = "大组ID不可为空"
		return
	}
	if req.TeamName == "" {
		br.Msg = "分组名称不能为空"
		return
	}
	teamNameArr := strings.Split(req.TeamName, ",")
	for _, v := range teamNameArr {
		count, err := system.GetSysTeamCount(req.GroupId, v)
		if err != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取数据失败,Err:" + err.Error()
			return
		}
		if count <= 0 {
			item := new(system.SysGroup)
			item.GroupName = v
			item.DepartmentId = req.DepartmentId
			item.ParentId = req.GroupId
			item.CreateTime = time.Now()
			groupId, e := system.AddSysGroup(item)
			if e != nil {
				br.Msg = "新增失败"
				br.ErrMsg = "新增失败，Err:" + e.Error()
				return
			}

			// 同步分组缓存
			var syncData system.SyncGroupData
			syncData.Source = utils.SOURCE_CRM_FLAG
			syncData.GroupId = int(groupId)
			_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)
		}
	}
	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}

// @Title 修改分组
// @Description  修改分组接口
// @Param	request	body system.SysTeamEditReq true "type json string"
// @Success 200 修改成功
// @router /team/edit [post]
func (this *SysTeamController) Edit() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysTeamEditReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.TeamId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "参数错误,GroupId 小于等于0 "
		return
	}
	if req.TeamName == "" {
		br.Msg = "分组名称不能为空"
		return
	}
	item, err := system.GetSysTeamByName(req.TeamName)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取数据失败,Err:" + err.Error()
		return
	}
	if item != nil {
		if item.GroupId != req.TeamId {
			br.Msg = "名称已存在，请重新输入"
			return
		}
	}
	err = system.ModifySysTeam(req.TeamName, req.TeamId)
	if err != nil {
		br.Msg = "修改失败"
		br.ErrMsg = "修改失败，Err:" + err.Error()
		return
	}

	// 同步分组缓存
	var syncData system.SyncGroupData
	syncData.Source = utils.SOURCE_CRM_FLAG
	syncData.GroupId = item.GroupId
	_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)

	br.Ret = 200
	br.Success = true
	br.Msg = "修改成功"
}

// @Title 删除分组
// @Description 删除分组接口
// @Param	request	body system.SysTeamDeleteReq true "type json string"
// @Success 200 删除成功
// @router /team/delete [post]
func (this *SysTeamController) Delete() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysTeamDeleteReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.TeamId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "参数错误,GroupId 小于等于0 "
		return
	}
	err = system.DeleteSysGroup(req.TeamId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败，Err:" + err.Error()
		return
	}
	err = system.ClearSysUserTeam(req.TeamId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败,情况分组失败，Err:" + err.Error()
		return
	}

	// 同步分组缓存
	var syncData system.SyncGroupData
	syncData.Source = utils.SOURCE_CRM_FLAG
	syncData.GroupId = req.TeamId
	_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)

	br.Ret = 200
	br.Success = true
	br.Msg = "删除成功"
}
