package data_manage

import (
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"hongze/hz_crm_api/utils"
	"time"
)

// BaseFromSmmClassify SMM原始数据分类表
type BaseFromSmmClassify struct {
	ClassifyId      int       `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

// AddBaseFromSmmClassify 添加SMM原始数据分类
func AddBaseFromSmmClassify(item *BaseFromSmmClassify) (lastId int64, err error) {
	o := orm.NewOrmUsingDB("data")
	lastId, err = o.Insert(item)
	return
}

// GetBaseFromSmmClassifyCount 获取分类名称的个数
func GetBaseFromSmmClassifyCount(classifyName string, parentId int) (count int, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT COUNT(1) AS count FROM base_from_smm_classify WHERE classify_name=? AND parent_id=? `
	err = o.Raw(sql, classifyName, parentId).QueryRow(&count)
	return
}

// GetBaseFromSmmClassifyById 通过分类id的获取分类信息
func GetBaseFromSmmClassifyById(classifyId int) (item *BaseFromSmmClassify, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `SELECT * FROM base_from_smm_classify WHERE classify_id=? `
	err = o.Raw(sql, classifyId).QueryRow(&item)
	return
}

// EditBaseFromSmmClassify 修改SMM原始数据分类
func EditBaseFromSmmClassify(classifyId int, classifyName string) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `UPDATE base_from_smm_classify SET classify_name=?,modify_time=NOW() WHERE classify_id=? `
	_, err = o.Raw(sql, classifyName, classifyId).Exec()
	return
}

// UpdateBaseFromSmmClassifySort 修改SMM原始数据分类的排序
func UpdateBaseFromSmmClassifySort(classifyId int) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `UPDATE base_from_smm_classify SET sort=classify_id, modify_time=NOW() WHERE classify_id=? `
	_, err = o.Raw(sql, classifyId).Exec()
	return
}

type BaseFromSmmClassifyItems struct {
	ClassifyId           int    `description:"分类ID"`
	BaseFromSmmIndexId   int    `description:"指标类型ID"`
	BaseFromSmmIndexCode string `description:"指标唯一编码"`
	ClassifyName         string `description:"分类名称"`
	ParentId             int    `description:"父级id"`
	Level                int    `description:"层级"`
	Sort                 int    `description:"排序字段，越小越靠前，默认值：10"`
	//UniqueCode         string                         `description:"唯一编码"`
	//ModifyTime         time.Time                      `description:"修改时间"`
	//CreateTime         time.Time                      `description:"创建时间"`
	Children []*BaseFromSmmClassifyItems `description:"下级"`
}

type BaseFromSmmClassifyNameItems struct {
	ClassifyId   int                             `description:"分类ID"`
	ClassifyName string                          `description:"分类名称"`
	ParentId     int                             `description:"父级id"`
	Children     []*BaseFromSmmClassifyNameItems `description:"下级"`
}

type BaseFromSmmClassifyResp struct {
	List []*BaseFromSmmClassifyItems
}

type BaseFromSmmClassifyNameResp struct {
	List []*BaseFromSmmClassifyNameItems
}

type BaseFromSmmClassifyItemsButton struct {
	AddButton    bool `description:"是否可添加"`
	OpButton     bool `description:"是否可编辑"`
	DeleteButton bool `description:"是否可删除"`
	MoveButton   bool `description:"是否可移动"`
}

// GetBaseFromSmmClassifyByParentId 根据上级id获取当下的分类列表数据
func GetBaseFromSmmClassifyByParentId(parentId int) (items []*BaseFromSmmClassifyItems, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT * FROM base_from_smm_classify WHERE parent_id=? order by sort asc,classify_id asc`
	_, err = o.Raw(sql, parentId).QueryRows(&items)
	return
}

// GetAllBaseFromSmmClassify 获取所有的分类列表数据
func GetAllBaseFromSmmClassify() (items []*BaseFromSmmClassifyItems, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT * FROM base_from_smm_classify WHERE parent_id <> 0 order by sort asc,classify_id asc`
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

type DeleteBaseFromSmmClassifyReq struct {
	ClassifyId int `description:"分类id"`
	EdbInfoId  int `description:"指标id"`
}

type BaseFromSmmClassifyListResp struct {
	AllNodes      []*BaseFromSmmClassifyItems
	CanOpClassify bool `description:"是否允许操作分类"`
}

type BaseFromSmmClassifySimplify struct {
	ClassifyId   int    `description:"分类id"`
	ClassifyName string `description:"分类名称"`
	ParentId     int
}

// GetFirstBaseFromSmmClassify 获取当前分类下，且排序数相同 的排序第一条的数据
func GetFirstBaseFromSmmClassify() (item *BaseFromSmmClassify, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT * FROM base_from_smm_classify order by sort asc,classify_id asc limit 1`
	err = o.Raw(sql).QueryRow(&item)
	return
}

// UpdateBaseFromSmmClassifySortByClassifyId 根据分类id更新排序
func UpdateBaseFromSmmClassifySortByClassifyId(parentId, classifyId, nowSort int, updateSort string) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` update base_from_smm_classify set sort = ` + updateSort + ` WHERE parent_id=? AND sort > ? `
	if classifyId > 0 {
		sql += ` or ( classify_id > ` + fmt.Sprint(classifyId) + ` and sort = ` + fmt.Sprint(nowSort) + `)`
	}
	_, err = o.Raw(sql, parentId, nowSort).Exec()
	return
}

// MoveUpSmmIndexClassifyBySort 往上移动
func MoveUpSmmIndexClassifyBySort(parentId, nextSort, currentSort int) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `update base_from_smm_classify set sort = sort + 1 where parent_id=? and sort >= ? and sort< ?`
	_, err = o.Raw(sql, parentId, nextSort, currentSort).Exec()
	return
}

// MoveDownSmmIndexClassifyBySort 往下移动
func MoveDownSmmIndexClassifyBySort(parentId, prevSort, currentSort int) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `update base_from_smm_classify set sort = sort - 1 where parent_id=? and sort <= ? and sort> ? `
	_, err = o.Raw(sql, parentId, prevSort, currentSort).Exec()
	return
}

// GetSmmIndexClassifyMinSort 获取最小不等于0的排序
func GetSmmIndexClassifyMinSort(parentId int) (sort int, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `select min(sort) from base_from_smm_classify where parent_id=? and sort <> 0 `
	err = o.Raw(sql, parentId).QueryRow(&sort)
	return
}

// Update 更新分类基础信息
func (BaseFromSmmClassify *BaseFromSmmClassify) Update(cols []string) (err error) {
	o := orm.NewOrmUsingDB("data")
	_, err = o.Update(BaseFromSmmClassify, cols...)
	return
}

type AddSmmClassifyResp struct {
	ClassifyId int
}

// DeleteSmmClassifyByClassifyId 根据分类id删除对应的指标分类
func DeleteSmmClassifyByClassifyId(classifyIdList []int) (err error) {
	o := orm.NewOrmUsingDB("data")
	num := len(classifyIdList)
	if num <= 0 {
		return
	}
	//删除分类
	sql := `DELETE FROM base_from_smm_classify WHERE classify_id IN (` + utils.GetOrmInReplace(num) + `) `
	_, err = o.Raw(sql, classifyIdList).Exec()
	return
}

// AddSmmClassifyMulti 批量新增SMM类别
func AddSmmClassifyMulti(list []*BaseFromSmmClassify) (err error) {
	o := orm.NewOrmUsingDB("data")
	_, err = o.InsertMulti(1, list)
	return
}

// InitSmmClassifySort 初始化sort值
func InitSmmClassifySort() (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `UPDATE base_from_smm_classify 
SET modify_time=NOW(), sort = classify_id`
	_, err = o.Raw(sql).Exec()
	return
}

// InitSmmIndexClassifyId 历史数据的classifyId值
func InitSmmIndexClassifyId() (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := `UPDATE base_from_smm_index s
LEFT JOIN (
SELECT
	c1.classify_id,
	CONCAT( c2.classify_name, c1.classify_name ) AS type_name 
FROM
	base_from_smm_classify c1
	LEFT JOIN base_from_smm_classify c2 ON c1.parent_id = c2.classify_id 
	) AS t ON CONCAT( s.type_2, s.type_3 ) = t.type_name
	SET s.classify_id = t.classify_id, s.modify_time=NOW() where s.type_2 <>""`
	_, err = o.Raw(sql).Exec()
	return
}
