package data_manage

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hz_crm_api/utils"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateHcz struct {
	EdbInfoCalculateHczId int       `orm:"column(edb_info_calculate_hcz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
}

// 环差值
func AddCalculateHcz(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string, formulaInt int) (edbInfoId int, err error) {
	fmt.Println("AddCalculateHcz")
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("AddCalculateHcz,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	if req.EdbInfoId <= 0 {
		edbInfo := new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_HCZ
		edbInfo.SourceName = "环差值"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}
		edbInfoId = int(newEdbInfoId)

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfoId = req.EdbInfoId
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_HCZ)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
	}

	edbInfoIdStr := strconv.Itoa(edbInfoId)
	fmt.Println("edbInfoIdStr:" + edbInfoIdStr)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	if req.EdbInfoId <= 0 {
		pars = append(pars, req.FromEdbInfoId)
	} else {
		pars = append(pars, fromEdbInfo.EdbInfoId)
	}
	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return edbInfoId, err
	}

	addSql := ` INSERT INTO edb_data_calculate_hcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existMap := make(map[string]string)
	dataLen := len(dataList)
	fmt.Println("dataLen:", dataLen)
	for i := 0; i < dataLen; i++ {
		j := i + formulaInt
		if j < dataLen {
			//当期
			currentItem := dataList[i]
			preItem := dataList[j]
			if currentItem != nil && preItem != nil {
				existKey := edbCode + currentItem.DataTime
				if _, ok := existMap[existKey]; !ok {
					currentDate, _ := time.Parse(utils.FormatDate, currentItem.DataTime)
					timestamp := currentDate.UnixNano() / 1e6
					timestampStr := fmt.Sprintf("%d", timestamp)
					val := ""
					if preItem.Value == 0 {
						val = "0"
					} else {
						val = HczDiv(currentItem.Value, preItem.Value)
					}
					if val != "" {
						addSql += GetAddSql(edbInfoIdStr, edbCode, currentItem.DataTime, timestampStr, val)
						isAdd = true
					}
				}
				existMap[existKey] = currentItem.DataTime
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return edbInfoId, err
		}
	}
	return
}

// 环差值，current:当期,pre:上期 公式:当期-上期
func HczDiv(current, pre float64) string {
	if pre == 0 {
		return ""
	}
	currentVal := decimal.NewFromFloat(current)
	preVal := decimal.NewFromFloat(pre)
	val, _ := currentVal.Sub(preVal).Float64()
	valStr := utils.SubFloatToString(val, 4)
	return valStr
}

// 环比值
func EditCalculateHcz(req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo, edbCode string, formulaInt int, calculateFormula string) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//修改指标信息
	sql := ` UPDATE  edb_info
			SET
			  edb_name =?,
			  edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?,
 			  calculate_formula=?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, req.Formula, edbInfoId).Exec()
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	existCondition += " AND from_edb_info_id=? "
	existPars = append(existPars, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(req.Source, existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}

	if count <= 0 || strconv.Itoa(formulaInt) != calculateFormula {
		//删除，计算指标关联的，基础指标的关联关系
		if count <= 0 {
			sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
			_, err = to.Raw(sql, edbInfoId).Exec()
			if err != nil {
				return edbInfoId, err
			}
			//关联关系
			{
				calculateMappingItem := new(EdbInfoCalculateMapping)
				calculateMappingItem.CreateTime = time.Now()
				calculateMappingItem.ModifyTime = time.Now()
				calculateMappingItem.Sort = 1
				calculateMappingItem.EdbCode = edbCode
				calculateMappingItem.EdbInfoId = edbInfoId
				calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
				calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
				calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
				calculateMappingItem.FromSource = fromEdbInfo.Source
				calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
				calculateMappingItem.FromTag = ""
				calculateMappingItem.Source = utils.DATA_SOURCE_CALCULATE_HCZ
				calculateMappingItem.SourceName = "环差值"
				_, err = to.Insert(calculateMappingItem)
				if err != nil {
					return
				}
			}
		}
		//清空原有数据
		sql = ` DELETE FROM edb_data_calculate_hcz WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}
		edbInfoIdStr := strconv.Itoa(edbInfoId)
		fmt.Println("edbInfoIdStr:" + edbInfoIdStr)
		//计算数据
		var condition string
		var pars []interface{}
		condition += " AND edb_info_id=? "
		pars = append(pars, req.FromEdbInfoId)
		fmt.Println("EdbInfoId:", req.FromEdbInfoId)
		dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
		if err != nil {
			return edbInfoId, err
		}
		var dateArr []string
		dataMap := make(map[string]*EdbInfoSearchData)
		for _, v := range dataList {
			dateArr = append(dateArr, v.DataTime)
			dataMap[v.DataTime] = v
		}

		addSql := ` INSERT INTO edb_data_calculate_hcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
		var isAdd bool
		existMap := make(map[string]string)
		dataLen := len(dataList)
		fmt.Println("dataLen:", dataLen)
		for i := 0; i < dataLen; i++ {
			j := i + formulaInt
			if j < dataLen {
				//当期
				currentItem := dataList[i]
				preItem := dataList[j]
				if currentItem != nil && preItem != nil {
					existKey := edbCode + currentItem.DataTime
					if _, ok := existMap[existKey]; !ok {
						currentDate, _ := time.Parse(utils.FormatDate, currentItem.DataTime)
						timestamp := currentDate.UnixNano() / 1e6
						timestampStr := fmt.Sprintf("%d", timestamp)
						val := HczDiv(currentItem.Value, preItem.Value)
						if val != "" {
							addSql += GetAddSql(edbInfoIdStr, edbCode, currentItem.DataTime, timestampStr, val)
							isAdd = true
						}
					}
					existMap[existKey] = currentItem.DataTime
				}
			}
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			_, err = to.Raw(addSql).Exec()
			if err != nil {
				return edbInfoId, err
			}
		}
	}
	return
}

// 刷新环比值数据
func RefreshCalculateHcz(edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}

	//获取指标所有数据
	existDataList := make([]*EdbDataBase, 0)
	dataTableName := GetEdbDataTableName(source)
	fmt.Println("dataTableName:", dataTableName)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	existDataMap := make(map[string]string)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v.Value
	}
	addSql := ` INSERT INTO edb_data_calculate_hcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existMap := make(map[string]string)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		j := i + 1
		if j < dataLen {
			//当期
			currentItem := dataList[i]
			preItem := dataList[j]
			if currentItem != nil && preItem != nil {
				existKey := edbCode + currentItem.DataTime
				if _, ok := existMap[existKey]; !ok {
					currentDate, _ := time.Parse(utils.FormatDate, currentItem.DataTime)
					timestamp := currentDate.UnixNano() / 1e6
					timestampStr := fmt.Sprintf("%d", timestamp)
					val := HczDiv(currentItem.Value, preItem.Value)
					if val != "" {
						if existVal, findOk := existDataMap[currentItem.DataTime]; !findOk {
							addSql += GetAddSql(edbInfoIdStr, edbCode, currentItem.DataTime, timestampStr, val)
							isAdd = true
						} else {
							if existVal != val {
								sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
								sql = fmt.Sprintf(sql, dataTableName)
								_, err = to.Raw(sql, val, edbInfoId, currentItem.DataTime).Exec()
								if err != nil {
									return err
								}
							}
						}
					}
				}
				existMap[existKey] = currentItem.DataTime
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateHczDetail struct {
	EdbInfoCalculateHczId int       `orm:"column(edb_info_calculate_hcz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
	StartDate             string    `description:"开始日期"`
	EndDate               string    `description:"结束日期"`
}

func GetEdbInfoCalculateHczDetail(edbInfoId int) (item *EdbInfoCalculateHczDetail, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

// 刷新所有环差值数据
func RefreshAllCalculateHcz(edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string, formulaInt int) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}
	fmt.Println("source:", source)
	//获取指标所有数据
	existDataList := make([]*EdbDataBase, 0)
	dataTableName := GetEdbDataTableName(source)
	fmt.Println("dataTableName:", dataTableName)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	existDataMap := make(map[string]string)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v.Value
	}
	fmt.Println("existDataMap:", existDataMap)
	addSql := ` INSERT INTO edb_data_calculate_hcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existMap := make(map[string]string)
	dataLen := len(dataList)
	fmt.Println("dataLen:", dataLen)
	for i := 0; i < dataLen; i++ {
		j := i + formulaInt
		if j < dataLen {
			//当期
			currentItem := dataList[i]
			preItem := dataList[j]
			if currentItem != nil && preItem != nil {
				existKey := edbCode + currentItem.DataTime
				if _, ok := existMap[existKey]; !ok {
					currentDate, _ := time.Parse(utils.FormatDate, currentItem.DataTime)
					timestamp := currentDate.UnixNano() / 1e6
					timestampStr := fmt.Sprintf("%d", timestamp)
					val := HczDiv(currentItem.Value, preItem.Value)
					if val != "" {
						if existVal, findOk := existDataMap[currentItem.DataTime]; !findOk {
							addSql += GetAddSql(edbInfoIdStr, edbCode, currentItem.DataTime, timestampStr, val)
							isAdd = true
						} else {
							if existVal != val {
								sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
								sql = fmt.Sprintf(sql, dataTableName)
								_, err = to.Raw(sql, val, edbInfoId, currentItem.DataTime).Exec()
								if err != nil {
									return err
								}
							}
						}
					}
				}
				existMap[existKey] = currentItem.DataTime
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}
