package data_manage

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hz_crm_api/utils"
	"sort"
	"strings"
	"time"
)

// AddCalculateLjztbpj 新增累计值同比拼接数据
func AddCalculateLjztbpj(req *EdbInfoCalculateBatchSaveReq, firstEdbInfo, secondEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfoId int, err error) {
	fmt.Println("AddCalculateLjztbpj")
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("AddCalculateLjztbpj,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	//待拼接指标map
	pjDataMap := make(map[string]float64)

	//拼接指标的日期切片数据
	pjEdbDataTimeList := make([]string, 0)

	//最近开始的时间
	var lastDateTime time.Time
	//获取待拼接指标
	{
		var condition string
		var pars []interface{}

		//获取待拼接指标最近的个12月31日有值的年份
		condition += " AND data_time like ? AND edb_info_id=? "
		pars = append(pars, "%12-31", firstEdbInfo.EdbInfoId)

		lastEdbData, tmpErr := GetLastEdbData(condition, pars, firstEdbInfo.Source)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}
		lastDateTime, _ = time.ParseInLocation(utils.FormatDate, lastEdbData.DataTime, time.Local)

		//获取待拼接指标的数据列表
		condition = ``
		pars = make([]interface{}, 0)
		condition += " AND data_time <= ? AND edb_info_id=? "
		pars = append(pars, lastEdbData.DataTime, firstEdbInfo.EdbInfoId)

		firstDataList, tmpErr := GetEdbDataListAll(condition, pars, firstEdbInfo.Source, 0)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}

		for _, v := range firstDataList {
			pjDataMap[v.DataTime] = v.Value
			pjEdbDataTimeList = append(pjEdbDataTimeList, v.DataTime)
		}
	}

	var edbInfo *EdbInfo
	if req.EdbInfoId <= 0 {
		edbInfo = &EdbInfo{
			SourceName:       "累计值同比拼接",
			Source:           utils.DATA_SOURCE_CALCULATE_LJZTBPJ,
			EdbCode:          edbCode,
			EdbName:          req.EdbName,
			EdbNameSource:    req.EdbName,
			Frequency:        req.Frequency,
			Unit:             req.Unit,
			StartDate:        firstEdbInfo.StartDate,
			EndDate:          firstEdbInfo.EndDate,
			ClassifyId:       req.ClassifyId,
			SysUserId:        sysUserId,
			SysUserRealName:  sysUserRealName,
			UniqueCode:       uniqueCode,
			CreateTime:       time.Now(),
			ModifyTime:       time.Now(),
			CalculateFormula: lastDateTime.Format(utils.FormatDate),
			EdbType:          2,
		}
		newEdbInfoId, err := to.Insert(edbInfo)
		if err != nil {
			return edbInfoId, err
		}
		edbInfoId = int(newEdbInfoId)
	} else {
		edbInfoId = req.EdbInfoId
		//查询
		tmpEdbInfo, tmpErr := GetEdbInfoById(edbInfoId)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		tmpEdbInfo.EdbName = req.EdbName
		tmpEdbInfo.ClassifyId = req.ClassifyId
		tmpEdbInfo.Frequency = req.Frequency
		tmpEdbInfo.Unit = req.Unit
		tmpEdbInfo.CalculateFormula = req.Formula

		edbInfo = tmpEdbInfo

		//删除指标数据
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_LJZTBPJ)
		fmt.Println("dataTableName:" + dataTableName)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
		if err != nil {
			return 0, err
		}

		//删除指标关系
		sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id=? `
		_, err = to.Raw(sql, edbInfoId).Exec()
	}

	//关联关系

	//拼接指标
	{
		calculateMappingItem := new(EdbInfoCalculateMapping)
		calculateMappingItem.CreateTime = time.Now()
		calculateMappingItem.ModifyTime = time.Now()
		calculateMappingItem.Sort = 1
		calculateMappingItem.EdbCode = edbCode
		calculateMappingItem.EdbInfoId = edbInfoId
		calculateMappingItem.FromEdbInfoId = firstEdbInfo.EdbInfoId
		calculateMappingItem.FromEdbCode = firstEdbInfo.EdbCode
		calculateMappingItem.FromEdbName = firstEdbInfo.EdbName
		calculateMappingItem.FromSource = firstEdbInfo.Source
		calculateMappingItem.FromSourceName = firstEdbInfo.SourceName
		calculateMappingItem.FromTag = "A"
		calculateMappingItem.Source = edbInfo.Source
		calculateMappingItem.SourceName = edbInfo.SourceName
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//同比值指标
	{
		calculateMappingItem := new(EdbInfoCalculateMapping)
		calculateMappingItem.CreateTime = time.Now()
		calculateMappingItem.ModifyTime = time.Now()
		calculateMappingItem.Sort = 1
		calculateMappingItem.EdbCode = edbCode
		calculateMappingItem.EdbInfoId = edbInfoId
		calculateMappingItem.FromEdbInfoId = secondEdbInfo.EdbInfoId
		calculateMappingItem.FromEdbCode = secondEdbInfo.EdbCode
		calculateMappingItem.FromEdbName = secondEdbInfo.EdbName
		calculateMappingItem.FromSource = secondEdbInfo.Source
		calculateMappingItem.FromSourceName = secondEdbInfo.SourceName
		calculateMappingItem.FromTag = "B"
		calculateMappingItem.Source = edbInfo.Source
		calculateMappingItem.SourceName = edbInfo.SourceName
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//同比值指标map
	tbzEdbDataMap := make(map[string]float64)

	//同比值日期切片列表
	tbzEdbDataTimeList := make([]string, 0)

	//同比值指标
	{
		var condition string
		var pars []interface{}

		condition += " AND data_time > ? AND edb_info_id = ? "
		pars = append(pars, lastDateTime, secondEdbInfo.EdbInfoId)

		//同比值指标的数据列表
		secondDataList, tmpErr := GetEdbDataListAll(condition, pars, secondEdbInfo.Source, 0)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}

		for _, v := range secondDataList {
			tbzEdbDataMap[v.DataTime] = v.Value
			tbzEdbDataTimeList = append(tbzEdbDataTimeList, v.DataTime)
		}
	}

	sort.Strings(tbzEdbDataTimeList)
	for _, v := range tbzEdbDataTimeList {
		tbzDataTime, _ := time.ParseInLocation(utils.FormatDate, v, time.Local)

		//获取拼接指标上一年同一天的数据
		var pjDataTime time.Time
		if tbzDataTime.Month() == 2 {
			pjDataTime = tbzDataTime.AddDate(0, -11, 0)
			pjDataTime = time.Date(pjDataTime.Year(), pjDataTime.Month(), 1, 0, 0, 0, 0, time.Now().Location()).AddDate(0, 0, -1)
		} else {
			pjDataTime = tbzDataTime.AddDate(-1, 0, 0)
		}

		//如果存在数据，那么就去计算当前的数据
		if pjDataValue, ok := pjDataMap[pjDataTime.Format(utils.FormatDate)]; ok {
			tbzDataValue := tbzEdbDataMap[v] //同比值
			currValue := pjDataValue * (1 + tbzDataValue/100)

			currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64()
			//将计算后的数据存入待拼接指标map里面，以便后续计算
			pjDataMap[v] = currValue

			pjEdbDataTimeList = append(pjEdbDataTimeList, v)
		}
	}

	addDataList := make([]*EdbDataCalculateLjztbpj, 0)

	//日期排序下
	sort.Strings(pjEdbDataTimeList)
	//这么做的目的是为了让数据插入的时候，可以正序插入（业务上没啥卵用，就是为了让我看数据的时候舒服点，手动狗头-_-|）
	for _, dataTime := range pjEdbDataTimeList {
		if dataValue, ok := pjDataMap[dataTime]; ok {
			//时间戳
			currentDate, _ := time.Parse(utils.FormatDate, dataTime)
			timestamp := currentDate.UnixNano() / 1e6

			edbDataLjztbpj := &EdbDataCalculateLjztbpj{
				EdbInfoId:     edbInfoId,
				EdbCode:       edbInfo.EdbCode,
				DataTime:      dataTime,
				Value:         dataValue,
				Status:        1,
				CreateTime:    time.Now(),
				ModifyTime:    time.Now(),
				DataTimestamp: timestamp,
			}
			addDataList = append(addDataList, edbDataLjztbpj)
		}
	}

	//数据入库
	tmpAddDataList := make([]*EdbDataCalculateLjztbpj, 0)
	for _, v := range addDataList {
		tmpAddDataList = append(tmpAddDataList, v)

		if len(tmpAddDataList) >= 200 {
			_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
			if tmpErr != nil {
				err = tmpErr
				return
			}
			//重新初始化需要加入的数据切片
			tmpAddDataList = make([]*EdbDataCalculateLjztbpj, 0)
		}
	}
	//最后如果还有需要新增的数据，那么就统一入库
	if len(tmpAddDataList) > 0 {
		_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
		if tmpErr != nil {
			err = tmpErr
			return
		}
	}

	return
}

// EditCalculateLjztbpj 编辑累计值同比拼接数据
func EditCalculateLjztbpj(req *EdbInfoCalculateBatchEditReq, nowEdbInfo, firstEdbInfo, secondEdbInfo *EdbInfo) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	sql := ``

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	//查询出所有的关联指标
	existList, err := GetEdbInfoCalculateListByCondition(existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}

	var existItemA, existItemB *EdbInfoCalculateMapping
	for _, existItem := range existList {
		if existItem.FromTag == "A" {
			existItemA = existItem
		} else if existItem.FromTag == "B" {
			existItemB = existItem
		}
	}
	// 原数据开始计算日期
	startCalculationDate, _ := time.ParseInLocation(utils.FormatDate, nowEdbInfo.CalculateFormula, time.Local)

	//待拼接指标map
	pjDataMap := make(map[string]float64)     //需要入库的数据
	nowEdbDataMap := make(map[string]float64) //当前指标的数据（已经在库里了，不需要重新）

	//拼接指标的日期切片数据
	pjEdbDataTimeList := make([]string, 0)

	//最近开始的时间
	lastDateTime := startCalculationDate

	//待拼接指标数据
	{
		//如果拼接指标变更了，那么需要删除所有的指标进行重新拼接
		if existItemA.FromEdbInfoId != firstEdbInfo.EdbInfoId {
			//删除旧的指标数据
			{
				//删除之前的A指标关联关系
				sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? and from_edb_info_id = ?`
				_, err = to.Raw(sql, edbInfoId, existItemA.FromEdbInfoId).Exec()
				if err != nil {
					err = errors.New("删除拼接日期之前的指标关联关系失败,Err:" + err.Error())
					return
				}

				//如果拼接指标变更了，那么需要删除所有的指标数据
				tableName := GetEdbDataTableName(nowEdbInfo.Source)
				sql = fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ? `, tableName)

				_, err = to.Raw(sql, edbInfoId).Exec()
				if err != nil {
					err = errors.New("删除所有的累计值同比拼接指标数据失败,Err:" + err.Error())
					return
				}
			}

			//添加新的指标关系
			{
				calculateMappingItem := new(EdbInfoCalculateMapping)
				calculateMappingItem.CreateTime = time.Now()
				calculateMappingItem.ModifyTime = time.Now()
				calculateMappingItem.Sort = 1
				calculateMappingItem.EdbCode = nowEdbInfo.EdbCode
				calculateMappingItem.EdbInfoId = edbInfoId
				calculateMappingItem.FromEdbInfoId = firstEdbInfo.EdbInfoId
				calculateMappingItem.FromEdbCode = firstEdbInfo.EdbCode
				calculateMappingItem.FromEdbName = firstEdbInfo.EdbName
				calculateMappingItem.FromSource = firstEdbInfo.Source
				calculateMappingItem.FromSourceName = firstEdbInfo.SourceName
				calculateMappingItem.FromTag = "A"
				calculateMappingItem.Source = nowEdbInfo.Source
				calculateMappingItem.SourceName = nowEdbInfo.SourceName
				_, err = to.Insert(calculateMappingItem)
				if err != nil {
					return
				}
			}

			var condition string
			var pars []interface{}

			//获取待拼接指标最近的个12月31日有值的年份
			condition += " AND data_time like ? AND edb_info_id=? "
			pars = append(pars, "%12-31", firstEdbInfo.EdbInfoId)

			//获取最新的待拼接指标
			lastEdbData, tmpErr := GetLastEdbData(condition, pars, firstEdbInfo.Source)
			if tmpErr != nil {
				return edbInfoId, tmpErr
			}
			lastDateTime, _ = time.ParseInLocation(utils.FormatDate, lastEdbData.DataTime, time.Local)

			//获取待拼接指标的数据列表
			condition = ``
			pars = make([]interface{}, 0)
			condition += " AND data_time <= ? AND edb_info_id=? "
			pars = append(pars, lastEdbData.DataTime, firstEdbInfo.EdbInfoId)

			firstDataList, tmpErr := GetEdbDataListAll(condition, pars, firstEdbInfo.Source, 0)
			if tmpErr != nil {
				return edbInfoId, tmpErr
			}

			for _, v := range firstDataList {
				pjDataMap[v.DataTime] = v.Value
				pjEdbDataTimeList = append(pjEdbDataTimeList, v.DataTime)

				//将新的数据存入已入库指标map里面，以便后续计算
				nowEdbDataMap[v.DataTime] = v.Value
			}
		} else {
			// 获取当前指标的所有数据
			var condition string
			var pars []interface{}
			condition += " AND edb_info_id=? and data_time <= ?"
			pars = append(pars, nowEdbInfo.EdbInfoId, nowEdbInfo.CalculateFormula)

			nowEdbDataList, tmpErr := GetEdbDataListAll(condition, pars, nowEdbInfo.Source, 0)
			if tmpErr != nil {
				return edbInfoId, tmpErr
			}

			for _, v := range nowEdbDataList {
				nowEdbDataMap[v.DataTime] = v.Value
				pjEdbDataTimeList = append(pjEdbDataTimeList, v.DataTime)
			}
		}
	}

	//同比值指标map
	tbzEdbDataMap := make(map[string]float64)

	//同比值日期切片列表
	tbzEdbDataTimeList := make([]string, 0)

	//同比值指标
	{
		if existItemB.FromEdbInfoId != secondEdbInfo.EdbInfoId {
			//删除通过B指标生成的数据
			{
				//删除之前的B指标关联关系
				sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? and from_edb_info_id = ?`
				_, err = to.Raw(sql, edbInfoId, existItemB.FromEdbInfoId).Exec()
				if err != nil {
					err = errors.New("删除拼接日期之后的指标关联关系失败,Err:" + err.Error())
					return
				}

				//如果同比值指标变更了，那么需要删除所有计算出来的指标数据
				tableName := GetEdbDataTableName(nowEdbInfo.Source)
				sql = fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ? and data_time > ? `, tableName)

				_, err = to.Raw(sql, edbInfoId, nowEdbInfo.CalculateFormula).Exec()
				if err != nil {
					err = errors.New("删除计算出来的累计值同比拼接指标数据失败,Err:" + err.Error())
					return
				}
			}

			//添加新的指标关系
			{
				calculateMappingItem := new(EdbInfoCalculateMapping)
				calculateMappingItem.CreateTime = time.Now()
				calculateMappingItem.ModifyTime = time.Now()
				calculateMappingItem.Sort = 1
				calculateMappingItem.EdbCode = nowEdbInfo.EdbCode
				calculateMappingItem.EdbInfoId = edbInfoId
				calculateMappingItem.FromEdbInfoId = secondEdbInfo.EdbInfoId
				calculateMappingItem.FromEdbCode = secondEdbInfo.EdbCode
				calculateMappingItem.FromEdbName = secondEdbInfo.EdbName
				calculateMappingItem.FromSource = secondEdbInfo.Source
				calculateMappingItem.FromSourceName = secondEdbInfo.SourceName
				calculateMappingItem.FromTag = "B"
				calculateMappingItem.Source = nowEdbInfo.Source
				calculateMappingItem.SourceName = nowEdbInfo.SourceName
				_, err = to.Insert(calculateMappingItem)
				if err != nil {
					return
				}
			}
		}

		var condition string
		var pars []interface{}

		condition += " AND data_time > ? AND edb_info_id = ? "
		pars = append(pars, lastDateTime, secondEdbInfo.EdbInfoId)

		//同比值指标的数据列表
		secondDataList, tmpErr := GetEdbDataListAll(condition, pars, secondEdbInfo.Source, 0)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}

		for _, v := range secondDataList {
			tbzEdbDataMap[v.DataTime] = v.Value
			tbzEdbDataTimeList = append(tbzEdbDataTimeList, v.DataTime)
		}
	}

	sort.Strings(tbzEdbDataTimeList)

	// 遍历现有的数据，判断拼接指标中是否存在该日期数据，如果拼接指标无此数据，那么需要删除该日期数据（日期的判断：需要在开始计算日期之后）
	removeDateList := make([]string, 0)
	for nowEdbDate := range nowEdbDataMap {
		nowEdbDateTime, _ := time.ParseInLocation(utils.FormatDate, nowEdbDate, time.Local)
		//校验日期 需要 大于 拼接前日期
		if startCalculationDate.Before(nowEdbDateTime) {
			if _, ok := tbzEdbDataMap[nowEdbDate]; !ok {
				// 同比指标中，不存在该日期数据，那么需要移除 现有数据 中该日期的数据
				removeDateList = append(removeDateList, nowEdbDate)
			}
		}
	}

	//待修改的指标数据map（index：日期，value：值）
	updateEdbDataMap := make(map[string]float64)
	for _, v := range tbzEdbDataTimeList {
		tbzDataTime, _ := time.ParseInLocation(utils.FormatDate, v, time.Local)

		//获取拼接指标上一年同一天的数据
		var pjDataTime time.Time
		if tbzDataTime.Month() == 2 {
			pjDataTime = tbzDataTime.AddDate(0, -11, 0)
			pjDataTime = time.Date(pjDataTime.Year(), pjDataTime.Month(), 1, 0, 0, 0, 0, time.Now().Location()).AddDate(0, 0, -1)
		} else {
			pjDataTime = tbzDataTime.AddDate(-1, 0, 0)
		}

		//校验现有数据中，是否存在该日期的数据，如果存在的话，那么就要去校验 最新计算数据 与 现有数据 是否一致
		if nowEdbDataValue, isHas := nowEdbDataMap[v]; isHas {
			//获取去年今日的数据，获取到后，然后是去修改该日期的数据
			if lastYearEdbDataValue, ok := nowEdbDataMap[pjDataTime.Format(utils.FormatDate)]; ok {
				tbzDataValue := tbzEdbDataMap[v] //同比值
				currValue := lastYearEdbDataValue * (1 + tbzDataValue/100)
				currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64() //保留4位小数
				//如果计算出来的值与库里面的值不匹配，那么就去修改该值
				if nowEdbDataValue != currValue {
					//将计算后的数据存入待拼接指标map里面，以便后续计算
					updateEdbDataMap[v] = currValue
				}
			}
		} else {
			//因为 现有数据中 不存在该日期数据，那么需要做新增数据处理

			//如果去年今日存在该数据，那么就去计算当前的数据
			if pjDataValue, ok := nowEdbDataMap[pjDataTime.Format(utils.FormatDate)]; ok {
				tbzDataValue := tbzEdbDataMap[v] //同比值
				currValue := pjDataValue * (1 + tbzDataValue/100)

				currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64()
				//将计算后的数据存入已入库指标map里面，以便后续计算
				nowEdbDataMap[v] = currValue

				//将计算后的数据存入待拼接指标map里面，以便后续入库
				pjDataMap[v] = currValue
				pjEdbDataTimeList = append(pjEdbDataTimeList, v)
			}
		}
	}

	//新增的数据入库
	{
		addDataList := make([]*EdbDataCalculateLjztbpj, 0)
		for dataTime, dataValue := range pjDataMap {
			//时间戳
			currentDate, _ := time.Parse(utils.FormatDate, dataTime)
			timestamp := currentDate.UnixNano() / 1e6

			edbDataLjztbpj := &EdbDataCalculateLjztbpj{
				EdbInfoId:     edbInfoId,
				EdbCode:       nowEdbInfo.EdbCode,
				DataTime:      dataTime,
				Value:         dataValue,
				Status:        1,
				CreateTime:    time.Now(),
				ModifyTime:    time.Now(),
				DataTimestamp: timestamp,
			}
			addDataList = append(addDataList, edbDataLjztbpj)
		}

		tmpAddDataList := make([]*EdbDataCalculateLjztbpj, 0)
		for _, v := range addDataList {
			tmpAddDataList = append(tmpAddDataList, v)

			if len(tmpAddDataList) >= 200 {
				_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
				if tmpErr != nil {
					err = tmpErr
					return
				}
				//重新初始化需要加入的数据切片
				tmpAddDataList = make([]*EdbDataCalculateLjztbpj, 0)
			}
		}
		//最后如果还有需要新增的数据，那么就统一入库
		if len(tmpAddDataList) > 0 {
			_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
			if tmpErr != nil {
				err = tmpErr
				return
			}
		}
	}

	//删除已经不存在的累计同比拼接指标数据（由于同比值当日的数据删除了）
	{
		if len(removeDateList) > 0 {
			removeDateStr := strings.Join(removeDateList, `","`)
			removeDateStr = `"` + removeDateStr + `"`
			//如果拼接指标变更了，那么需要删除所有的指标数据
			tableName := GetEdbDataTableName(nowEdbInfo.Source)
			sql = fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ? and data_time in (%s) `, tableName, removeDateStr)

			_, err = to.Raw(sql, edbInfoId).Exec()
			if err != nil {
				err = errors.New("删除不存在的累计值同比拼接指标数据失败,Err:" + err.Error())
				return
			}
		}
	}

	//修改现有的数据中对应的值
	{
		tableName := GetEdbDataTableName(nowEdbInfo.Source)
		for edbDate, edbDataValue := range updateEdbDataMap {
			sql = fmt.Sprintf(` UPDATE  %s set value = %f,modify_time=now() WHERE edb_info_id = ? and data_time = %s `, tableName, edbDataValue, edbDate)

			_, err = to.Raw(sql, edbInfoId).Exec()
			if err != nil {
				err = errors.New("更新现有的累计值同比拼接指标数据失败,Err:" + err.Error())
				return
			}
		}
	}

	//修改指标信息
	sql = ` UPDATE  edb_info SET
			  edb_name =?,
			  edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?, 
			  calculate_formula=?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, lastDateTime.Format(utils.FormatDate), edbInfoId).Exec()
	if err != nil {
		return
	}

	return
}

// RefreshAllCalculateLjztbpj 刷新所有 累计值同比拼接 数据
func RefreshAllCalculateLjztbpj(edbInfo *EdbInfo) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//查询当前指标现有的数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, edbInfo.EdbInfoId)

	//所有的数据
	dataList, err := GetAllEdbDataCalculateLjztbpjByEdbInfoId(edbInfo.EdbInfoId)
	if err != nil {
		return err
	}

	//待拼接指标map
	pjDataMap := make(map[string]float64)     //需要入库的数据
	nowEdbDataMap := make(map[string]float64) //当前指标的数据（已经在库里了，不需要重新）
	//拼接指标的日期切片数据
	pjEdbDataTimeList := make([]string, 0)

	dataMap := make(map[string]*EdbDataCalculateLjztbpj)
	for _, v := range dataList {
		pjEdbDataTimeList = append(pjEdbDataTimeList, v.DataTime)
		dataMap[v.DataTime] = v
		nowEdbDataMap[v.DataTime] = v.Value
	}

	//查询关联指标信息
	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfo.EdbInfoId)
	existList, err := GetEdbInfoCalculateListByCondition(existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}

	var existItemA, existItemB *EdbInfoCalculateMapping
	for _, existItem := range existList {
		if existItem.FromTag == "A" {
			existItemA = existItem
		} else if existItem.FromTag == "B" {
			existItemB = existItem
		}
	}
	// 原数据开始计算日期
	startCalculationDate, _ := time.ParseInLocation(utils.FormatDate, edbInfo.CalculateFormula, time.Local)

	//待拼接指标
	{
		var condition string
		var pars []interface{}

		condition += " AND data_time <= ? AND edb_info_id=? "
		pars = append(pars, startCalculationDate, existItemA.FromEdbInfoId)

		//第一个指标的数据列表
		firstDataList, tmpErr := GetEdbDataListAll(condition, pars, existItemA.FromSource, 0)
		if tmpErr != nil {
			return tmpErr
		}

		for _, v := range firstDataList {
			//时间戳
			if edbData, ok := dataMap[v.DataTime]; ok {
				if edbData.Value != v.Value {
					//更新指标数据
					edbData.Value = v.Value
					_, err = to.Update(edbData, "Value")
					if err != nil {
						return
					}

					//将新的数据存入已入库指标map里面，以便后续计算
					nowEdbDataMap[edbData.DataTime] = v.Value
				}
			}
		}
	}

	//同比值指标map
	tbzEdbDataMap := make(map[string]float64)

	//同比值日期切片列表
	tbzEdbDataTimeList := make([]string, 0)

	//同比值指标
	{
		var condition string
		var pars []interface{}

		condition += " AND data_time > ? AND edb_info_id = ? "
		pars = append(pars, startCalculationDate, existItemB.FromEdbInfoId)

		//第二个指标的数据列表
		secondDataList, tmpErr := GetEdbDataListAll(condition, pars, existItemB.FromSource, 0)
		if tmpErr != nil {
			return tmpErr
		}

		for _, v := range secondDataList {
			tbzEdbDataMap[v.DataTime] = v.Value
			tbzEdbDataTimeList = append(tbzEdbDataTimeList, v.DataTime)
		}
	}

	sort.Strings(tbzEdbDataTimeList)

	// 遍历现有的数据，判断拼接指标中是否存在该日期数据，如果拼接指标无此数据，那么需要删除该日期数据（日期的判断：需要在开始计算日期之后）
	removeDateList := make([]string, 0)
	for nowEdbDate := range nowEdbDataMap {
		nowEdbDateTime, _ := time.ParseInLocation(utils.FormatDate, nowEdbDate, time.Local)
		//校验日期 需要 大于 拼接前日期
		if startCalculationDate.Before(nowEdbDateTime) {
			if _, ok := tbzEdbDataMap[nowEdbDate]; !ok {
				// 同比指标中，不存在该日期数据，那么需要移除 现有数据 中该日期的数据
				removeDateList = append(removeDateList, nowEdbDate)
			}
		}
	}

	//待修改的指标数据map（index：日期，value：值）
	updateEdbDataMap := make(map[string]float64)
	for _, v := range tbzEdbDataTimeList {
		tbzDataTime, _ := time.ParseInLocation(utils.FormatDate, v, time.Local)

		//获取拼接指标上一年同一天的数据
		var pjDataTime time.Time
		if tbzDataTime.Month() == 2 {
			pjDataTime = tbzDataTime.AddDate(0, -11, 0)
			pjDataTime = time.Date(pjDataTime.Year(), pjDataTime.Month(), 1, 0, 0, 0, 0, time.Now().Location()).AddDate(0, 0, -1)
		} else {
			pjDataTime = tbzDataTime.AddDate(-1, 0, 0)
		}

		//校验现有数据中，是否存在该日期的数据，如果存在的话，那么就要去校验 最新计算数据 与 现有数据 是否一致
		if nowEdbDataValue, isHas := nowEdbDataMap[v]; isHas {
			//获取去年今日的数据，获取到后，然后是去修改该日期的数据
			if lastYearEdbDataValue, ok := nowEdbDataMap[pjDataTime.Format(utils.FormatDate)]; ok {

				if v == "2021-08-31" {
					fmt.Println("进来了")
				}
				tbzDataValue := tbzEdbDataMap[v] //同比值
				currValue := lastYearEdbDataValue * (1 + tbzDataValue/100)
				currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64() //保留4位小数
				//如果计算出来的值与库里面的值不匹配，那么就去修改该值
				if nowEdbDataValue != currValue {
					//将计算后的数据存入待拼接指标map里面，以便后续计算
					updateEdbDataMap[v] = currValue
				}
			}
		} else {
			//因为 现有数据中 不存在该日期数据，那么需要做新增数据处理

			//如果去年今日存在该数据，那么就去计算当前的数据
			if pjDataValue, ok := nowEdbDataMap[pjDataTime.Format(utils.FormatDate)]; ok {
				tbzDataValue := tbzEdbDataMap[v] //同比值
				currValue := pjDataValue * (1 + tbzDataValue/100)

				currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64()
				//将计算后的数据存入已入库指标map里面，以便后续计算
				nowEdbDataMap[v] = currValue

				//将计算后的数据存入待拼接指标map里面，以便后续入库
				pjDataMap[v] = currValue
				pjEdbDataTimeList = append(pjEdbDataTimeList, v)
			}
		}
	}

	//新增的数据入库
	{
		addDataList := make([]*EdbDataCalculateLjztbpj, 0)
		for dataTime, dataValue := range pjDataMap {
			//时间戳
			currentDate, _ := time.Parse(utils.FormatDate, dataTime)
			timestamp := currentDate.UnixNano() / 1e6

			edbDataLjztbpj := &EdbDataCalculateLjztbpj{
				EdbInfoId:     edbInfo.EdbInfoId,
				EdbCode:       edbInfo.EdbCode,
				DataTime:      dataTime,
				Value:         dataValue,
				Status:        1,
				CreateTime:    time.Now(),
				ModifyTime:    time.Now(),
				DataTimestamp: timestamp,
			}
			addDataList = append(addDataList, edbDataLjztbpj)
		}

		tmpAddDataList := make([]*EdbDataCalculateLjztbpj, 0)
		for _, v := range addDataList {
			tmpAddDataList = append(tmpAddDataList, v)

			if len(tmpAddDataList) >= 200 {
				_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
				if tmpErr != nil {
					err = tmpErr
					return
				}
				//重新初始化需要加入的数据切片
				tmpAddDataList = make([]*EdbDataCalculateLjztbpj, 0)
			}
		}
		//最后如果还有需要新增的数据，那么就统一入库
		if len(tmpAddDataList) > 0 {
			_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
			if tmpErr != nil {
				err = tmpErr
				return
			}
		}
	}

	//删除已经不存在的累计同比拼接指标数据（由于同比值当日的数据删除了）
	{
		if len(removeDateList) > 0 {
			removeDateStr := strings.Join(removeDateList, `","`)
			removeDateStr = `"` + removeDateStr + `"`
			//如果拼接指标变更了，那么需要删除所有的指标数据
			tableName := GetEdbDataTableName(edbInfo.Source)
			sql := fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ? and data_time in (%s) `, tableName, removeDateStr)

			_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
			if err != nil {
				err = errors.New("删除不存在的累计值同比拼接指标数据失败,Err:" + err.Error())
				return
			}
		}
	}

	//修改现有的数据中对应的值
	{
		tableName := GetEdbDataTableName(edbInfo.Source)
		for edbDate, edbDataValue := range updateEdbDataMap {
			sql := fmt.Sprintf(` UPDATE  %s set value = ?,modify_time=now() WHERE edb_info_id = ? and data_time = ? `, tableName)

			_, err = to.Raw(sql, edbDataValue, edbInfo.EdbInfoId, edbDate).Exec()
			if err != nil {
				err = errors.New("更新现有的累计值同比拼接指标数据失败,Err:" + err.Error())
				return
			}
		}
	}
	return
}
