package data_manage

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hz_crm_api/utils"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateLjzzy struct {
	EdbInfoCalculateLjzzyId int       `orm:"column(edb_info_calculate_ljzzy_id);pk"`
	EdbInfoId               int       `description:"指标id"`
	EdbCode                 string    `description:"指标编码"`
	FromEdbInfoId           int       `description:"计算指标id"`
	FromEdbCode             string    `description:"计算指标编码"`
	FromEdbName             string    `description:"计算指标名称"`
	FromSource              int       `description:"计算指标来源"`
	FromSourceName          string    `description:"计算指标来源名称"`
	FromTag                 string    `description:"来源指标标签"`
	Sort                    int       `description:"计算指标名称排序"`
	CreateTime              time.Time `description:"创建时间"`
	ModifyTime              time.Time `description:"修改时间"`
}

func AddEdbInfoCalculateLjzzyMulti(items []*EdbInfoCalculateLjzzy) (err error) {
	o := orm.NewOrmUsingDB("data")
	_, err = o.InsertMulti(1, items)
	return
}

// 累计值转月
func AddCalculateLjzzy(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfoId int, err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if req.EdbInfoId <= 0 {
		edbInfo := new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_LJZZY
		edbInfo.SourceName = "累计值转月值"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			return int(newEdbInfoId), tmpErr
		}
		edbInfoId = int(newEdbInfoId)

		//calculateItem := new(EdbInfoCalculateLjzzy)
		//calculateItem.CreateTime = time.Now()
		//calculateItem.ModifyTime = time.Now()
		//calculateItem.Sort = 1
		//calculateItem.EdbCode = edbCode
		//calculateItem.EdbInfoId = edbInfoId
		//calculateItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		//calculateItem.FromEdbCode = fromEdbInfo.EdbCode
		//calculateItem.FromEdbName = fromEdbInfo.EdbName
		//calculateItem.FromSource = fromEdbInfo.Source
		//calculateItem.FromSourceName = fromEdbInfo.SourceName
		//_, err = to.Insert(calculateItem)
		//if err != nil {
		//	return edbInfoId, err
		//}
		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfoId = req.EdbInfoId
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_LJZZY)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	if req.EdbInfoId <= 0 {
		pars = append(pars, req.FromEdbInfoId)
	} else {
		pars = append(pars, fromEdbInfo.EdbInfoId)
	}
	fmt.Println("EdbInfoId:", req.FromEdbInfoId)
	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 1)
	if err != nil {
		return edbInfoId, err
	}

	yearMap := make(map[int]map[int]*EdbInfoSearchData)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		item := dataList[i]
		//日其中获取年
		itemDate, err := time.Parse(utils.FormatDate, item.DataTime)
		if err != nil {
			return edbInfoId, err
		}
		year := itemDate.Year()
		month := int(itemDate.Month())
		if monthMap, yok := yearMap[year]; yok {
			monthMap[month] = item
			yearMap[year] = monthMap
		} else {
			monthMap = make(map[int]*EdbInfoSearchData)
			monthMap[month] = item
			yearMap[year] = monthMap
		}
	}

	addSql := ` INSERT INTO edb_data_calculate_ljzzy(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existMap := make(map[string]string)
	for yk, yv := range yearMap {
		_, oneMonthOk := yv[1]
		_, twoMonthOk := yv[2]
		if !oneMonthOk && !twoMonthOk {
			continue
		}
		for i := 1; i <= 12; i++ {
			fmt.Println(yk, i, yv[i])
			dataCurrentItem := yv[i]
			fmt.Println("i:", i, yk, dataCurrentItem)
			var date string
			var val float64
			if i == 1 || i == 2 {
				if _, mok := yv[1]; mok { //1月有值
					if i == 1 {
						date = dataCurrentItem.DataTime
						val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
					}
					if i == 2 {
						dataOneItem := yv[1]
						if dataCurrentItem != nil {
							date = dataCurrentItem.DataTime
							twoMonth := decimal.NewFromFloat(dataCurrentItem.Value)
							oneMonth := decimal.NewFromFloat(dataOneItem.Value)
							val, _ = twoMonth.Sub(oneMonth).Float64()
						} else {
							continue
						}
					}
				} else { //1月无值
					dataTwoItem := yv[2]
					if i == 1 {
						date = strconv.Itoa(yk) + "-01-31"
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
					if i == 2 {
						date = dataCurrentItem.DataTime
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
				}
			} else {
				dataPreItem := yv[i-1]
				if dataCurrentItem != nil && dataPreItem != nil {
					date = dataCurrentItem.DataTime
					//val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Sub(decimal.NewFromFloat(dataPreItem.Value)).Float64()
					a := decimal.NewFromFloat(dataCurrentItem.Value)
					b := decimal.NewFromFloat(dataPreItem.Value)
					val, _ = a.Sub(b).Float64()
				}
			}

			if date != "" {
				dataTime, _ := time.Parse(utils.FormatDate, date)
				timestamp := dataTime.UnixNano() / 1e6
				timeStr := fmt.Sprintf("%d", timestamp)

				if _, ok := existMap[edbCode+date]; !ok {
					addSql += GetAddSql(edbInfoIdStr, edbCode, date, timeStr, utils.SubFloatToString(val, 4))
					isAdd = true
				}
				existMap[edbCode+date] = date
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return edbInfoId, err
		}
	}
	return
}

// 修改累计值转月
func EditCalculateLjzzy(req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo, edbCode string) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//修改指标信息
	sql := ` UPDATE  edb_info
			SET
			  edb_name =?,
              edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, edbInfoId).Exec()
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	existCondition += " AND from_edb_info_id=? "
	existPars = append(existPars, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(req.Source, existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count <= 0 {
		//删除指标关联计算指标
		//sql := ` DELETE FROM edb_info_calculate_ljzzy WHERE edb_info_id = ? `
		//_, err = to.Raw(sql, edbInfoId).Exec()
		//if err != nil {
		//	return
		//}

		//calculateItem := new(EdbInfoCalculateLjzzy)
		//calculateItem.CreateTime = time.Now()
		//calculateItem.ModifyTime = time.Now()
		//calculateItem.Sort = 1
		//calculateItem.EdbCode = edbCode
		//calculateItem.EdbInfoId = edbInfoId
		//calculateItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		//calculateItem.FromEdbCode = fromEdbInfo.EdbCode
		//calculateItem.FromEdbName = fromEdbInfo.EdbName
		//calculateItem.FromSource = fromEdbInfo.Source
		//calculateItem.FromSourceName = fromEdbInfo.SourceName
		//_, err = to.Insert(calculateItem)
		//if err != nil {
		//	return
		//}

		//删除，计算指标关联的，基础指标的关联关系
		sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return
		}

		//清空原有数据
		sql = ` DELETE FROM edb_data_calculate_ljzzy WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = utils.DATA_SOURCE_CALCULATE_LJZZY
			calculateMappingItem.SourceName = "累计值转月"
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}

		//计算数据
		var condition string
		var pars []interface{}
		condition += " AND edb_info_id=? "
		pars = append(pars, req.FromEdbInfoId)
		fmt.Println("EdbInfoId:", req.FromEdbInfoId)
		dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 1)
		if err != nil {
			return edbInfoId, err
		}

		yearMap := make(map[int]map[int]*EdbInfoSearchData)
		dataLen := len(dataList)
		for i := 0; i < dataLen; i++ {
			item := dataList[i]
			//日其中获取年
			itemDate, err := time.Parse(utils.FormatDate, item.DataTime)
			if err != nil {
				return edbInfoId, err
			}
			year := itemDate.Year()
			month := int(itemDate.Month())
			if monthMap, yok := yearMap[year]; yok {
				monthMap[month] = item
				yearMap[year] = monthMap
			} else {
				monthMap = make(map[int]*EdbInfoSearchData)
				monthMap[month] = item
				yearMap[year] = monthMap
			}
		}

		addSql := ` INSERT INTO edb_data_calculate_ljzzy(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
		nowStr := time.Now().Format(utils.FormatDateTime)
		var isAdd bool

		for yk, yv := range yearMap {
			_, oneMonthOk := yv[1]
			_, twoMonthOk := yv[2]
			if !oneMonthOk && !twoMonthOk {
				continue
			}
			for i := 1; i <= 12; i++ {
				fmt.Println(yk, i, yv[i])
				dataCurrentItem := yv[i]
				var date string
				var val float64
				if i == 1 || i == 2 {
					if _, mok := yv[1]; mok { //1月有值
						if i == 1 {
							date = dataCurrentItem.DataTime
							val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
						}
						if i == 2 {
							dataOneItem := yv[1]
							date = dataCurrentItem.DataTime
							twoMonth := decimal.NewFromFloat(dataCurrentItem.Value)
							oneMonth := decimal.NewFromFloat(dataOneItem.Value)
							val, _ = twoMonth.Sub(oneMonth).Float64()
						}
					} else { //1月无值
						dataTwoItem := yv[2]
						if i == 1 {
							date = strconv.Itoa(yk) + "-01-31"
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
						if i == 2 {
							date = dataCurrentItem.DataTime
							a := decimal.NewFromFloat(dataTwoItem.Value)
							b := decimal.NewFromFloat(2.0)
							val, _ = a.Div(b).Float64()
						}
					}
				} else {
					dataPreItem := yv[i-1]
					if dataCurrentItem != nil && dataPreItem != nil {
						date = dataCurrentItem.DataTime
						//val = dataCurrentItem.Value - dataPreItem.Value
						a := decimal.NewFromFloat(dataCurrentItem.Value)
						b := decimal.NewFromFloat(dataPreItem.Value)
						val, _ = a.Sub(b).Float64()
					}
				}
				if date != "" {
					dataTime, _ := time.Parse(utils.FormatDate, date)
					timestamp := dataTime.UnixNano() / 1e6
					timeStr := fmt.Sprintf("%d", timestamp)

					addSql += "("
					addSql += strconv.Itoa(edbInfoId) + "," + "'" + edbCode + "'" + "," + "'" + date + "'" + "," + utils.SubFloatToString(val, 4) + "," + "'" + nowStr + "'" +
						"," + "'" + nowStr + "'" + "," + "1"
					addSql += "," + "'" + timeStr + "'"
					addSql += "),"
					isAdd = true
				}
			}
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			_, err = to.Raw(addSql).Exec()
			if err != nil {
				return edbInfoId, err
			}
		}
	}
	return
}

// 刷新累计值转月值数据
func RefreshCalculateLjzzy(edbInfoId int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 1)
	if err != nil {
		return err
	}

	yearMap := make(map[int]map[int]*EdbInfoSearchData)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		item := dataList[i]
		//日其中获取年
		itemDate, err := time.Parse(utils.FormatDate, item.DataTime)
		if err != nil {
			return err
		}
		year := itemDate.Year()
		month := int(itemDate.Month())
		if monthMap, yok := yearMap[year]; yok {
			monthMap[month] = item
			yearMap[year] = monthMap
		} else {
			monthMap = make(map[int]*EdbInfoSearchData)
			monthMap[month] = item
			yearMap[year] = monthMap
		}
	}

	addSql := ` INSERT INTO edb_data_calculate_ljzzy(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	nowStr := time.Now().Format(utils.FormatDateTime)
	var isAdd bool

	for yk, yv := range yearMap {
		_, oneMonthOk := yv[1]
		_, twoMonthOk := yv[2]
		if !oneMonthOk && !twoMonthOk {
			continue
		}
		for i := 1; i <= 12; i++ {
			fmt.Println(yk, i, yv[i])
			dataCurrentItem := yv[i]
			var date string
			var val float64
			if i == 1 || i == 2 {
				if _, mok := yv[1]; mok { //1月有值
					if i == 1 {
						date = dataCurrentItem.DataTime
						val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
					}
					if i == 2 {
						dataOneItem := yv[1]
						date = dataCurrentItem.DataTime
						twoMonth := decimal.NewFromFloat(dataCurrentItem.Value)
						oneMonth := decimal.NewFromFloat(dataOneItem.Value)
						val, _ = twoMonth.Sub(oneMonth).Float64()
					}
				} else { //1月无值
					dataTwoItem := yv[2]
					if i == 1 {
						date = strconv.Itoa(yk) + "-01-31"
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
					if i == 2 {
						date = dataCurrentItem.DataTime
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
				}
			} else {
				dataPreItem := yv[i-1]
				if dataCurrentItem != nil && dataPreItem != nil {
					date = dataCurrentItem.DataTime
					//val =  dataCurrentItem.Value - dataPreItem.Value
					a := decimal.NewFromFloat(dataCurrentItem.Value)
					b := decimal.NewFromFloat(dataPreItem.Value)
					val, _ = a.Sub(b).Float64()
				}
			}
			if date != "" {
				//判断数据是否存在
				count, err := GetEdbDataCalculateLjzzyByCodeAndDate(edbCode, date)
				if err != nil && err.Error() != utils.ErrNoRow() {
					return err
				}
				if count <= 0 {
					dataTime, _ := time.Parse(utils.FormatDate, date)
					timestamp := dataTime.UnixNano() / 1e6
					timeStr := fmt.Sprintf("%d", timestamp)

					addSql += "("
					addSql += edbInfoIdStr + "," + "'" + edbCode + "'" + "," + "'" + date + "'" + "," + utils.SubFloatToString(val, 4) + "," + "'" + nowStr + "'" +
						"," + "'" + nowStr + "'" + "," + "1"
					addSql += "," + "'" + timeStr + "'"
					addSql += "),"
					isAdd = true
				} else {
					valStr := utils.SubFloatToString(val, 4)
					err = ModifyEdbDataCalculateLjzzy(int64(edbInfoId), date, valStr)
					if err != nil {
						return err
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateLjzzyDetail struct {
	EdbInfoCalculateLjzzyId int       `orm:"column(edb_info_calculate_ljzzy_id);pk"`
	EdbInfoId               int       `description:"指标id"`
	EdbCode                 string    `description:"指标编码"`
	FromEdbInfoId           int       `description:"计算指标id"`
	FromEdbCode             string    `description:"计算指标编码"`
	FromEdbName             string    `description:"计算指标名称"`
	FromSource              int       `description:"计算指标来源"`
	FromSourceName          string    `description:"计算指标来源名称"`
	FromTag                 string    `description:"来源指标标签"`
	Sort                    int       `description:"计算指标名称排序"`
	CreateTime              time.Time `description:"创建时间"`
	ModifyTime              time.Time `description:"修改时间"`
	StartDate               string    `description:"开始日期"`
	EndDate                 string    `description:"结束日期"`
}

func GetEdbInfoCalculateLjzzyDetail(edbInfoId int) (item *EdbInfoCalculateLjzzyDetail, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

func GetEdbDataCalculateLjzzyByCodeAndDate(edbCode string, startDate string) (count int, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_data_calculate_ljzzy WHERE edb_code=? AND data_time=? `
	err = o.Raw(sql, edbCode, startDate).QueryRow(&count)
	return
}

func ModifyEdbDataCalculateLjzzy(edbInfoId int64, dataTime, value string) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` UPDATE edb_data_calculate_ljzzy SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
	_, err = o.Raw(sql, value, edbInfoId, dataTime).Exec()
	return
}

// 刷新全部数据
func RefreshAllCalculateLjzzy(edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 1)
	if err != nil {
		return err
	}

	yearMap := make(map[int]map[int]*EdbInfoSearchData)
	dataLen := len(dataList)
	for i := 0; i < dataLen; i++ {
		item := dataList[i]
		//日其中获取年
		itemDate, err := time.Parse(utils.FormatDate, item.DataTime)
		if err != nil {
			return err
		}
		year := itemDate.Year()
		month := int(itemDate.Month())
		if monthMap, yok := yearMap[year]; yok {
			monthMap[month] = item
			yearMap[year] = monthMap
		} else {
			monthMap = make(map[int]*EdbInfoSearchData)
			monthMap[month] = item
			yearMap[year] = monthMap
		}
	}

	addSql := ` INSERT INTO edb_data_calculate_ljzzy(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	//获取指标所有数据
	existDataList := make([]*EdbDataBase, 0)
	dataTableName := GetEdbDataTableName(source)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	dataMap := make(map[string]string)
	for _, v := range existDataList {
		dataMap[v.DataTime] = v.Value
	}
	existDataMap := make(map[string]string)
	for yk, yv := range yearMap {
		_, oneMonthOk := yv[1]
		_, twoMonthOk := yv[2]
		if !oneMonthOk && !twoMonthOk {
			continue
		}
		for i := 1; i <= 12; i++ {
			fmt.Println(yk, i, yv[i])
			dataCurrentItem := yv[i]
			var date string
			var val float64
			if i == 1 || i == 2 {
				if _, mok := yv[1]; mok { //1月有值
					if i == 1 {
						date = dataCurrentItem.DataTime
						val, _ = decimal.NewFromFloat(dataCurrentItem.Value).Float64() //a.Div(b).Float64()
					}
					if i == 2 {
						dataOneItem := yv[1]
						date = dataCurrentItem.DataTime
						twoMonth := decimal.NewFromFloat(dataCurrentItem.Value)
						oneMonth := decimal.NewFromFloat(dataOneItem.Value)
						val, _ = twoMonth.Sub(oneMonth).Float64()
					}
				} else { //1月无值
					dataTwoItem := yv[2]
					if i == 1 {
						date = strconv.Itoa(yk) + "-01-31"
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
					if i == 2 {
						date = dataCurrentItem.DataTime
						a := decimal.NewFromFloat(dataTwoItem.Value)
						b := decimal.NewFromFloat(2.0)
						val, _ = a.Div(b).Float64()
					}
				}
			} else {
				dataPreItem := yv[i-1]
				if dataCurrentItem != nil && dataPreItem != nil {
					date = dataCurrentItem.DataTime
					//val =  dataCurrentItem.Value - dataPreItem.Value
					a := decimal.NewFromFloat(dataCurrentItem.Value)
					b := decimal.NewFromFloat(dataPreItem.Value)
					val, _ = a.Sub(b).Float64()
				}
			}
			if date != "" {
				saveValue := utils.SubFloatToString(val, 4)
				//判断数据是否存在
				if existVal, ok := dataMap[date]; !ok {
					dataTime, _ := time.Parse(utils.FormatDate, date)
					timestamp := dataTime.UnixNano() / 1e6
					timeStr := fmt.Sprintf("%d", timestamp)
					if _, existOk := existDataMap[date]; !existOk {
						addSql += GetAddSql(edbInfoIdStr, edbCode, date, timeStr, saveValue)
						isAdd = true
					}
					existDataMap[date] = date
				} else {
					if existVal != saveValue {
						sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
						sql = fmt.Sprintf(sql, dataTableName)
						_, err = to.Raw(sql, saveValue, edbInfoId, date).Exec()
						if err != nil {
							return err
						}
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshAllCalculateLjzzy add Err", err.Error())
			return
		}
	}
	return
}
