package data_manage

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hz_crm_api/utils"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateNszydpjjs struct {
	EdbInfoCalculateNszydpjjsId int       `orm:"column(edb_info_calculate_nszydpjjs_id);pk"`
	EdbInfoId                   int       `description:"指标id"`
	EdbCode                     string    `description:"指标编码"`
	FromEdbInfoId               int       `description:"计算指标id"`
	FromEdbCode                 string    `description:"计算指标编码"`
	FromEdbName                 string    `description:"计算指标名称"`
	FromSource                  int       `description:"计算指标来源"`
	FromSourceName              string    `description:"计算指标来源名称"`
	FromTag                     string    `description:"来源指标标签"`
	Sort                        int       `description:"计算指标名称排序"`
	CreateTime                  time.Time `description:"创建时间"`
	ModifyTime                  time.Time `description:"修改时间"`
}

// N数值移动平均计算
func AddCalculateNszydpjjs(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string, formulaInt int) (edbInfoId int, err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	if req.EdbInfoId <= 0 {
		edbInfo := new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_NSZYDPJJS
		edbInfo.SourceName = "N数值移动平均计算"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}
		edbInfoId = int(newEdbInfoId)

		//calculateItem := new(EdbInfoCalculateNszydpjjs)
		//calculateItem.CreateTime = time.Now()
		//calculateItem.ModifyTime = time.Now()
		//calculateItem.Sort = 1
		//calculateItem.EdbCode = edbCode
		//calculateItem.EdbInfoId = edbInfoId
		//calculateItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		//calculateItem.FromEdbCode = fromEdbInfo.EdbCode
		//calculateItem.FromEdbName = fromEdbInfo.EdbName
		//calculateItem.FromSource = fromEdbInfo.Source
		//calculateItem.FromSourceName = fromEdbInfo.SourceName
		//
		//_, err = to.Insert(calculateItem)
		//if err != nil {
		//	return edbInfoId, err
		//}

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfoId = req.EdbInfoId
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_NSZYDPJJS)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
	}

	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	if req.EdbInfoId <= 0 {
		pars = append(pars, req.FromEdbInfoId)
	} else {
		pars = append(pars, fromEdbInfo.EdbInfoId)
	}
	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return edbInfoId, err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_nszydpjjs(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool

	arrLen := len(dateArr)
	for ak, av := range dateArr {
		//处理第一个值
		var valArr []float64
		if findItem, ok := dataMap[av]; ok {
			valArr = append(valArr, findItem.Value)
		} else {
			continue
		}
		if ak+1 != arrLen {
			//处理除第一个值之外的N-1个值
			for i := 1; i < formulaInt; i++ {
				arrIndex := ak + i
				if arrIndex >= arrLen {
					break
				}
				arrVal := dateArr[arrIndex]
				if findItem, ok := dataMap[arrVal]; ok {
					valArr = append(valArr, findItem.Value)
				} else {
					continue
				}
			}
		}
		valArrLen := len(valArr)
		totalVal := decimal.NewFromFloat(0.00)
		for _, v := range valArr {
			newDecimal := decimal.NewFromFloat(v)
			totalVal = totalVal.Add(newDecimal)
		}
		af := totalVal //decimal.NewFromFloat(totalVal)
		bf := decimal.NewFromFloat(float64(valArrLen))
		val, _ := af.Div(bf).Float64()
		currentDate, err := time.Parse(utils.FormatDate, av)
		if err != nil {
			return edbInfoId, err
		}

		timestamp := currentDate.UnixNano() / 1e6
		timestampStr := fmt.Sprintf("%d", timestamp)
		valStr := utils.SubFloatToString(val, 4)
		addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, valStr)
		isAdd = true
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return edbInfoId, err
		}
	}
	return
}

// 修改N数值移动平均计算
func EditCalculateNszydpjjs(req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo, edbCode string, formulaInt int, oldCalculateFormula string) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId

	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	//修改指标信息
	sql := ` UPDATE  edb_info
			SET
			  edb_name =?,
              edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?,
			  calculate_formula=?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, req.Formula, edbInfoId).Exec()
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	existCondition += " AND from_edb_info_id=? "
	existPars = append(existPars, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(req.Source, existCondition, existPars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count <= 0 || oldCalculateFormula != req.Formula {
		//删除，计算指标关联的，基础指标的关联关系
		sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}
		//清空原有数据
		sql = ` DELETE FROM edb_data_calculate_nszydpjjs WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}
		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = utils.DATA_SOURCE_CALCULATE_NSZYDPJJS
			calculateMappingItem.SourceName = "N数值移动平均计算"
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}

		edbInfoIdStr := strconv.Itoa(edbInfoId)
		//计算数据
		var condition string
		var pars []interface{}
		condition += " AND edb_info_id=? "
		pars = append(pars, req.FromEdbInfoId)
		dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
		if err != nil {
			return edbInfoId, err
		}
		var dateArr []string
		dataMap := make(map[string]*EdbInfoSearchData)
		for _, v := range dataList {
			dateArr = append(dateArr, v.DataTime)
			dataMap[v.DataTime] = v
		}

		addSql := ` INSERT INTO edb_data_calculate_nszydpjjs(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
		var isAdd bool

		arrLen := len(dateArr)
		for ak, av := range dateArr {
			//处理第一个值
			var valArr []float64
			if findItem, ok := dataMap[av]; ok {
				valArr = append(valArr, findItem.Value)
			} else {
				continue
			}
			if ak+1 != arrLen {
				//处理除第一个值之外的N-1个值
				for i := 1; i < formulaInt; i++ {
					arrIndex := ak + i
					if arrIndex >= arrLen {
						break
					}
					arrVal := dateArr[arrIndex]
					if findItem, ok := dataMap[arrVal]; ok {
						valArr = append(valArr, findItem.Value)
					} else {
						continue
					}
				}
			}
			valArrLen := len(valArr)
			totalVal := decimal.NewFromFloat(0.00)
			for _, v := range valArr {
				newDecimal := decimal.NewFromFloat(v)
				totalVal = totalVal.Add(newDecimal)
			}
			af := totalVal
			bf := decimal.NewFromFloat(float64(valArrLen))
			val, _ := af.Div(bf).Float64()
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return edbInfoId, err
			}

			timestamp := currentDate.UnixNano() / 1e6
			timestampStr := fmt.Sprintf("%d", timestamp)

			valStr := utils.SubFloatToString(val, 4)
			addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, valStr)
			isAdd = true
		}

		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			_, err = to.Raw(addSql).Exec()
			if err != nil {
				return edbInfoId, err
			}
		}
	}
	return
}

// 刷新N数值移动平均计算
func RefreshCalculateNszydpjjs(edbInfoId, source, formulaInt int, fromEdbInfo *EdbInfo, edbCode string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("RefreshCalculateNszydpjjs Err:" + err.Error())
			utils.FileLog.Info("RefreshCalculateNszydpjjs Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, edbInfoId)

	existDataList, err := GetEdbDataListAll(condition, pars, source, 0)
	if err != nil {
		fmt.Println("existDataList GetEdbDataListAll Err:" + err.Error())
		return err
	}

	existDataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v
	}
	startDate := time.Now().AddDate(-2, 0, 0).Format(utils.FormatDate)
	//计算来源数据
	var fromCondition string
	var fromPars []interface{}
	fromCondition += " AND edb_info_id=? "
	fromPars = append(fromPars, fromEdbInfo.EdbInfoId)
	fromCondition += " AND data_time>=? "
	fromPars = append(fromPars, startDate)

	fromDataList, err := GetEdbDataListAll(fromCondition, fromPars, fromEdbInfo.Source, 0)
	if err != nil {
		fmt.Println("from GetEdbDataListAll Err:" + err.Error())
		return err
	}

	var fromDateArr []string
	fromDataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range fromDataList {
		fromDateArr = append(fromDateArr, v.DataTime)
		fromDataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_nszydpjjs(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool

	arrLen := len(fromDateArr)
	for ak, av := range fromDateArr {
		if _, existOk := existDataMap[av]; !existOk {
			//处理第一个值
			var valArr []float64
			if findItem, ok := fromDataMap[av]; ok {
				valArr = append(valArr, findItem.Value)
			} else {
				continue
			}
			if ak+1 != arrLen {
				//处理除第一个值之外的N-1个值
				for i := 1; i < formulaInt; i++ {
					arrIndex := ak + i
					if arrIndex >= arrLen {
						break
					}
					arrVal := fromDateArr[arrIndex]
					if findItem, ok := fromDataMap[arrVal]; ok {
						valArr = append(valArr, findItem.Value)
					} else {
						continue
					}
				}
			}
			valArrLen := len(valArr)
			//var totalVal float64
			totalVal := decimal.NewFromFloat(0.00)
			for _, v := range valArr {
				newDecimal := decimal.NewFromFloat(v)
				totalVal = totalVal.Add(newDecimal)
			}
			af := totalVal //decimal.NewFromFloat(totalVal)
			bf := decimal.NewFromFloat(float64(valArrLen))
			val, _ := af.Div(bf).Float64()
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return err
			}

			timestamp := currentDate.UnixNano() / 1e6
			timestampStr := fmt.Sprintf("%d", timestamp)
			valStr := utils.SubFloatToString(val, 4)
			addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, valStr)
			isAdd = true
		}
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateNszydpjjsDetail struct {
	EdbInfoCalculateNszydpjjsId int       `orm:"column(edb_info_calculate_nszydpjjs_id);pk"`
	EdbInfoId                   int       `description:"指标id"`
	EdbCode                     string    `description:"指标编码"`
	FromEdbInfoId               int       `description:"计算指标id"`
	FromEdbCode                 string    `description:"计算指标编码"`
	FromEdbName                 string    `description:"计算指标名称"`
	FromSource                  int       `description:"计算指标来源"`
	FromSourceName              string    `description:"计算指标来源名称"`
	FromTag                     string    `description:"来源指标标签"`
	Sort                        int       `description:"计算指标名称排序"`
	CreateTime                  time.Time `description:"创建时间"`
	ModifyTime                  time.Time `description:"修改时间"`
	StartDate                   string    `description:"开始日期"`
	EndDate                     string    `description:"结束日期"`
}

func GetEdbInfoCalculateNszydpjjsDetail(edbInfoId int) (item *EdbInfoCalculateNszydpjjsDetail, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

// 刷新全部N数值移动平均计算
func RefreshAllCalculateNszydpjjs(edbInfoId, source, formulaInt int, fromEdbInfo *EdbInfo, edbCode, startDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("RefreshCalculateNszydpjjs Err:" + err.Error())
			utils.FileLog.Info("RefreshCalculateNszydpjjs Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	fmt.Println(edbInfoIdStr)
	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, edbInfoId)

	existDataList, err := GetEdbDataListAll(condition, pars, source, 0)
	if err != nil {
		fmt.Println("existDataList GetEdbDataListAll Err:" + err.Error())
		return err
	}

	existDataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v
	}

	//计算来源数据
	var fromCondition string
	var fromPars []interface{}
	fromCondition += " AND edb_info_id=? "
	fromPars = append(fromPars, fromEdbInfo.EdbInfoId)
	fromCondition += " AND data_time>=? "
	fromPars = append(fromPars, startDate)

	fmt.Println("fromPars:", fromPars)
	fromDataList, err := GetEdbDataListAll(fromCondition, fromPars, fromEdbInfo.Source, 0)
	if err != nil {
		fmt.Println("from GetEdbDataListAll Err:" + err.Error())
		return err
	}

	var fromDateArr []string
	fromDataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range fromDataList {
		fromDateArr = append(fromDateArr, v.DataTime)
		fromDataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_nszydpjjs(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	dataTableName := GetEdbDataTableName(source)
	arrLen := len(fromDateArr)
	existAddDataMap := make(map[string]string)
	for ak, av := range fromDateArr {
		//处理第一个值
		var valArr []float64
		if findItem, ok := fromDataMap[av]; ok {
			valArr = append(valArr, findItem.Value)
		} else {
			continue
		}
		if ak+1 != arrLen {
			//处理除第一个值之外的N-1个值
			for i := 1; i < formulaInt; i++ {
				arrIndex := ak + i
				if arrIndex >= arrLen {
					break
				}
				arrVal := fromDateArr[arrIndex]
				if findItem, ok := fromDataMap[arrVal]; ok {
					valArr = append(valArr, findItem.Value)
				} else {
					continue
				}
			}
		}
		valArrLen := len(valArr)
		//var totalVal float64
		totalVal := decimal.NewFromFloat(0.00)
		for _, v := range valArr {
			newDecimal := decimal.NewFromFloat(v)
			totalVal = totalVal.Add(newDecimal)
		}
		af := totalVal //decimal.NewFromFloat(totalVal)
		bf := decimal.NewFromFloat(float64(valArrLen))
		val, _ := af.Div(bf).Float64()
		valStr := utils.SubFloatToString(val, 4)

		if existVal, existOk := existDataMap[av]; !existOk {
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return err
			}
			timestamp := currentDate.UnixNano() / 1e6
			timestampStr := fmt.Sprintf("%d", timestamp)
			if _, existOk := existAddDataMap[av]; !existOk {
				addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, valStr)
				isAdd = true
			}
			existAddDataMap[av] = av
		} else {
			saveValue := utils.SubFloatToString(existVal.Value, 30)
			if saveValue != valStr {
				sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
				sql = fmt.Sprintf(sql, dataTableName)
				_, err = to.Raw(sql, val, edbInfoId, av).Exec()
				if err != nil {
					return err
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}
