package data_manage

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hz_crm_api/utils"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateTcz struct {
	EdbInfoCalculateTbzId int       `orm:"column(edb_info_calculate_tcz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
}

// 同差值
func AddCalculateTcz(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfoId int, err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("err:", err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	fmt.Println("req.EdbInfoId:", req.EdbInfoId)
	if req.EdbInfoId <= 0 {
		edbInfo := new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_TCZ
		edbInfo.SourceName = "同差值"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}
		edbInfoId = int(newEdbInfoId)

		//calculateItem := new(EdbInfoCalculateTcz)
		//calculateItem.CreateTime = time.Now()
		//calculateItem.ModifyTime = time.Now()
		//calculateItem.Sort = 1
		//calculateItem.EdbCode = edbCode
		//calculateItem.EdbInfoId = edbInfoId
		//calculateItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		//calculateItem.FromEdbCode = fromEdbInfo.EdbCode
		//calculateItem.FromEdbName = fromEdbInfo.EdbName
		//calculateItem.FromSource = fromEdbInfo.Source
		//calculateItem.FromSourceName = fromEdbInfo.SourceName
		//
		//_, err = to.Insert(calculateItem)
		//if err != nil {
		//	return edbInfoId, err
		//}

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}
	} else {
		edbInfoId = req.EdbInfoId
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_TCZ)
		fmt.Println("dataTableName:", dataTableName)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}
	}

	edbInfoIdStr := strconv.Itoa(edbInfoId)

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	if req.EdbInfoId <= 0 {
		pars = append(pars, req.FromEdbInfoId)
	} else {
		pars = append(pars, fromEdbInfo.EdbInfoId)
	}

	fmt.Println("EdbInfoId:", req.FromEdbInfoId)
	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return edbInfoId, err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}
	fmt.Println("Frequency:", fromEdbInfo.Frequency)
	addSql := ` INSERT INTO edb_data_calculate_tcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existMap := make(map[string]string)
	for _, av := range dateArr {
		//fmt.Println("dateArr:", ak, av)
		currentItem := dataMap[av]
		if currentItem != nil {
			//当前日期
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return edbInfoId, err
			}
			//上一年的日期
			preDate := currentDate.AddDate(-1, 0, 0)
			preDateStr := preDate.Format(utils.FormatDate)
			if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
				//dataTime, _ := time.Parse(utils.FormatDate, date)
				if _, ok := existMap[edbCode+av]; !ok {
					timestamp := currentDate.UnixNano() / 1e6
					timestampStr := fmt.Sprintf("%d", timestamp)

					val := TczSub(currentItem.Value, findItem.Value)
					addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
					isAdd = true
				}
				existMap[edbCode+av] = av
			} else {
				if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
					nextDateDay := preDate
					preDateDay := preDate
					for i := 0; i <= 35; i++ {
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //下一年同期->下一个月找到
							if _, ok := existMap[edbCode+av]; !ok {
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							}
							existMap[edbCode+av] = av
							break
						} else {
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								if _, ok := existMap[edbCode+av]; !ok {
									timestamp := currentDate.UnixNano() / 1e6
									timestampStr := fmt.Sprintf("%d", timestamp)
									val := TczSub(currentItem.Value, findItem.Value)

									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existMap[edbCode+av] = av
								break
							}
						}
						nextDateDay = nextDateDay.AddDate(0, 0, 1)
						preDateDay = preDateDay.AddDate(0, 0, -1)
					}
				} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
					if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
						if _, ok := existMap[edbCode+av]; !ok {
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)
							addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
							isAdd = true
						}
						existMap[edbCode+av] = av
						break
					}
				} else {
					nextDateDay := preDate
					preDateDay := preDate
					for i := 0; i < 35; i++ {
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							if _, ok := existMap[edbCode+av]; !ok {
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							}
							existMap[edbCode+av] = av
							break
						} else {
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								if _, ok := existMap[edbCode+av]; !ok {
									timestamp := currentDate.UnixNano() / 1e6
									timestampStr := fmt.Sprintf("%d", timestamp)
									val := TczSub(currentItem.Value, findItem.Value)

									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existMap[edbCode+av] = av
								break
							} else {

							}
						}
						nextDateDay = nextDateDay.AddDate(0, 0, 1)
						preDateDay = preDateDay.AddDate(0, 0, -1)
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return edbInfoId, err
		}
	}
	return
}

func TczSub(a, b float64) string {
	af := decimal.NewFromFloat(float64(a))
	fmt.Println(af)
	bf := decimal.NewFromFloat(float64(b))
	val, _ := af.Sub(bf).Float64()
	valStr := utils.SubFloatToString(val, 4)
	return valStr
}

// 同差值
func EditCalculateTcz(req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo, edbCode string) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("err:", err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//修改指标信息
	sql := ` UPDATE  edb_info
			SET
			  edb_name =?,
			  edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, edbInfoId).Exec()
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	existCondition += " AND from_edb_info_id=? "
	existPars = append(existPars, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(req.Source, existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count <= 0 {
		//删除指标关联计算指标
		//sql := ` DELETE FROM edb_info_calculate_tcz WHERE edb_info_id = ? `
		//_, err = to.Raw(sql, edbInfoId).Exec()
		//if err != nil {
		//	return
		//}
		//
		//calculateItem := new(EdbInfoCalculateTcz)
		//calculateItem.CreateTime = time.Now()
		//calculateItem.ModifyTime = time.Now()
		//calculateItem.Sort = 1
		//calculateItem.EdbCode = edbCode
		//calculateItem.EdbInfoId = edbInfoId
		//calculateItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
		//calculateItem.FromEdbCode = fromEdbInfo.EdbCode
		//calculateItem.FromEdbName = fromEdbInfo.EdbName
		//calculateItem.FromSource = fromEdbInfo.Source
		//calculateItem.FromSourceName = fromEdbInfo.SourceName
		//
		//_, err = to.Insert(calculateItem)
		//if err != nil {
		//	return
		//}

		//删除，计算指标关联的，基础指标的关联关系
		sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return
		}

		//清空原有数据
		sql = ` DELETE FROM edb_data_calculate_tcz WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = utils.DATA_SOURCE_CALCULATE_TCZ
			calculateMappingItem.SourceName = "同差值"
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}

		edbInfoIdStr := strconv.Itoa(edbInfoId)

		//计算数据
		var condition string
		var pars []interface{}
		condition += " AND edb_info_id=? "
		pars = append(pars, req.FromEdbInfoId)
		fmt.Println("EdbInfoId:", req.FromEdbInfoId)
		dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
		if err != nil {
			return edbInfoId, err
		}
		var dateArr []string
		dataMap := make(map[string]*EdbInfoSearchData)
		for _, v := range dataList {
			dateArr = append(dateArr, v.DataTime)
			dataMap[v.DataTime] = v
		}

		addSql := ` INSERT INTO edb_data_calculate_tcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
		var isAdd bool
		for ak, av := range dateArr {
			fmt.Println(ak, av)
			currentItem := dataMap[av]
			if currentItem != nil {
				//当前日期
				currentDate, err := time.Parse(utils.FormatDate, av)
				if err != nil {
					return edbInfoId, err
				}
				//上一年的日期
				preDate := currentDate.AddDate(-1, 0, 0)
				preDateStr := preDate.Format(utils.FormatDate)
				if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
					//dataTime, _ := time.Parse(utils.FormatDate, date)
					timestamp := currentDate.UnixNano() / 1e6
					timestampStr := fmt.Sprintf("%d", timestamp)

					val := TczSub(currentItem.Value, findItem.Value)
					addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
					isAdd = true
					utils.FileLog.Info("同期找到:" + av + ";" + preDateStr)
					continue
				} else {
					if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
						nextDateDay := preDate.AddDate(0, 1, 0)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)

						preDateDay := preDate.AddDate(0, -1, 0)
						preDateDayStr := preDateDay.Format(utils.FormatDate)

						for i := 0; i <= 6; i++ {
							if i >= 1 {
								nextDateDay = nextDateDay.AddDate(0, 0, i)
								nextDateDayStr = nextDateDay.Format(utils.FormatDate)
							}
							if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
								break
							} else {
								if i >= 1 {
									preDateDay = preDate.AddDate(0, 0, -i)
									preDateDayStr = nextDateDay.Format(utils.FormatDate)
								}
								if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
									timestamp := currentDate.UnixNano() / 1e6
									timestampStr := fmt.Sprintf("%d", timestamp)
									val := TczSub(currentItem.Value, findItem.Value)

									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
									break
								}
							}
						}
					} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
						if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)

							addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
							isAdd = true
							break
						}
					} else {
						nextDateDay := preDate.AddDate(0, 0, 1)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)

						preDateDay := preDate.AddDate(0, 0, -1)
						preDateDayStr := preDateDay.Format(utils.FormatDate)

						for i := 0; i < 35; i++ {
							if i >= 1 {
								nextDateDay = nextDateDay.AddDate(0, 0, i)
								nextDateDayStr = nextDateDay.Format(utils.FormatDate)
							}
							if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
								break
							} else {
								if i >= 1 {
									preDateDay = preDate.AddDate(0, 0, -i)
									preDateDayStr = nextDateDay.Format(utils.FormatDate)
								}
								if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
									timestamp := currentDate.UnixNano() / 1e6
									timestampStr := fmt.Sprintf("%d", timestamp)
									val := TczSub(currentItem.Value, findItem.Value)

									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
									break
								}
							}
						}
					}
				}
			}
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			_, err = to.Raw(addSql).Exec()
			if err != nil {
				return edbInfoId, err
			}
		}
	}
	return
}

// 刷新同差值数据
func RefreshCalculateTcz(edbInfoId int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_tcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	for ak, av := range dateArr {
		fmt.Println(ak, av)
		currentItem := dataMap[av]
		if currentItem != nil {
			//当前日期
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return err
			}
			//上一年的日期
			preDate := currentDate.AddDate(-1, 0, 0)
			preDateStr := preDate.Format(utils.FormatDate)
			if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
				//dataTime, _ := time.Parse(utils.FormatDate, date)
				timestamp := currentDate.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				val := TczSub(currentItem.Value, findItem.Value)

				count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
				if err != nil && err.Error() != utils.ErrNoRow() {
					return err
				}
				if count <= 0 {
					addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
					isAdd = true
				} else {
					err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
					if err != nil {
						return err
					}
				}
				utils.FileLog.Info("同期找到:" + av + ";" + preDateStr)
				continue
			} else {
				if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
					for i := 0; i <= 35; i++ {
						nextDateDay := preDate.AddDate(0, 0, 1)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)

							count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
							if err != nil && err.Error() != utils.ErrNoRow() {
								return err
							}
							if count <= 0 {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							} else {
								err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
								if err != nil {
									return err
								}
							}
							break
						} else {
							preDateDay := preDate.AddDate(0, 0, -1)
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
								if err != nil && err.Error() != utils.ErrNoRow() {
									return err
								}
								if count <= 0 {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								} else {
									err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
									if err != nil {
										return err
									}
								}
								break
							}
						}
					}
				} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
					if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
						timestamp := currentDate.UnixNano() / 1e6
						timestampStr := fmt.Sprintf("%d", timestamp)
						val := TczSub(currentItem.Value, findItem.Value)

						count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
						if err != nil && err.Error() != utils.ErrNoRow() {
							return err
						}
						if count <= 0 {
							addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
							isAdd = true
						} else {
							err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
							if err != nil {
								return err
							}
						}
						break
					}
				} else {
					for i := 0; i < 35; i++ {
						nextDateDay := preDate.AddDate(0, 0, 1)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)

							count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
							if err != nil && err.Error() != utils.ErrNoRow() {
								return err
							}
							if count <= 0 {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							} else {
								err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
								if err != nil {
									return err
								}
							}
							break
						} else {
							preDateDay := preDate.AddDate(0, 0, -1)
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								count, err := GetEdbDataCalculateTczByCodeAndDate(edbCode, av)
								if err != nil && err.Error() != utils.ErrNoRow() {
									return err
								}
								if count <= 0 {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								} else {
									err = ModifyEdbDataCalculateTcz(int64(edbInfoId), av, val)
									if err != nil {
										return err
									}
								}
								break
							}
						}
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateTczDetail struct {
	EdbInfoCalculateTczId int       `orm:"column(edb_info_calculate_tcz_id);pk"`
	EdbInfoId             int       `description:"指标id"`
	EdbCode               string    `description:"指标编码"`
	FromEdbInfoId         int       `description:"计算指标id"`
	FromEdbCode           string    `description:"计算指标编码"`
	FromEdbName           string    `description:"计算指标名称"`
	FromSource            int       `description:"计算指标来源"`
	FromSourceName        string    `description:"计算指标来源名称"`
	FromTag               string    `description:"来源指标标签"`
	Sort                  int       `description:"计算指标名称排序"`
	CreateTime            time.Time `description:"创建时间"`
	ModifyTime            time.Time `description:"修改时间"`
	StartDate             string    `description:"开始日期"`
	EndDate               string    `description:"结束日期"`
}

func GetEdbInfoCalculateTczDetail(edbInfoId int) (item *EdbInfoCalculateTczDetail, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.from_edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

func GetEdbDataCalculateTczByCodeAndDate(edbCode string, startDate string) (count int, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_data_calculate_tcz WHERE edb_code=? AND data_time=? `
	err = o.Raw(sql, edbCode, startDate).QueryRow(&count)
	return
}

func ModifyEdbDataCalculateTcz(edbInfoId int64, dataTime, value string) (err error) {
	o := orm.NewOrmUsingDB("data")
	sql := ` UPDATE edb_data_calculate_tcz SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
	_, err = o.Raw(sql, value, edbInfoId, dataTime).Exec()
	return
}

// 刷新全部同差值数据
func RefreshAllCalculateTcz(edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	if endDate != "" {
		condition += " AND data_time<=? "
		pars = append(pars, endDate)
	}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}
	//获取指标所有数据
	existDataList := make([]*EdbDataBase, 0)
	dataTableName := GetEdbDataTableName(source)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	existDataMap := make(map[string]string)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v.Value
	}

	addSql := ` INSERT INTO edb_data_calculate_tcz(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool
	existAddDataMap := make(map[string]string)
	for _, av := range dateArr {
		currentItem := dataMap[av]
		if currentItem != nil {
			//当前日期
			currentDate, err := time.Parse(utils.FormatDate, av)
			if err != nil {
				return err
			}
			//上一年的日期
			preDate := currentDate.AddDate(-1, 0, 0)
			preDateStr := preDate.Format(utils.FormatDate)
			if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
				//dataTime, _ := time.Parse(utils.FormatDate, date)
				timestamp := currentDate.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				val := TczSub(currentItem.Value, findItem.Value)

				if existVal, ok := existDataMap[av]; !ok {
					if _, existOk := existAddDataMap[av]; !existOk {
						addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
						isAdd = true
					}
					existAddDataMap[av] = av
				} else {
					if existVal != val {
						sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
						sql = fmt.Sprintf(sql, dataTableName)
						_, err = to.Raw(sql, val, edbInfoId, av).Exec()
						if err != nil {
							return err
						}
					}
				}
				utils.FileLog.Info("同期找到:" + av + ";" + preDateStr)
				continue
			} else {
				if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
					for i := 0; i <= 35; i++ {
						nextDateDay := preDate.AddDate(0, 0, 1)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)

							if existVal, ok := existDataMap[av]; !ok {
								if _, existOk := existAddDataMap[av]; !existOk {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existAddDataMap[av] = av
							} else {
								if existVal != val {
									sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
									sql = fmt.Sprintf(sql, dataTableName)
									_, err = to.Raw(sql, val, edbInfoId, av).Exec()
									if err != nil {
										return err
									}
								}
							}
							break
						} else {
							preDateDay := preDate.AddDate(0, 0, -1)
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								if existVal, ok := existDataMap[av]; !ok {
									if _, existOk := existAddDataMap[av]; !existOk {
										addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
										isAdd = true
									}
									existAddDataMap[av] = av
								} else {
									if existVal != val {
										sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
										sql = fmt.Sprintf(sql, dataTableName)
										_, err = to.Raw(sql, val, edbInfoId, av).Exec()
										if err != nil {
											return err
										}
									}
								}
								break
							}
						}
					}
				} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
					if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
						timestamp := currentDate.UnixNano() / 1e6
						timestampStr := fmt.Sprintf("%d", timestamp)
						val := TczSub(currentItem.Value, findItem.Value)

						if existVal, ok := existDataMap[av]; !ok {
							if _, existOk := existAddDataMap[av]; !existOk {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							}
							existAddDataMap[av] = av
						} else {
							if existVal != val {
								sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
								sql = fmt.Sprintf(sql, dataTableName)
								_, err = to.Raw(sql, val, edbInfoId, av).Exec()
								if err != nil {
									return err
								}
							}
						}
						break
					}
				} else {
					for i := 0; i < 35; i++ {
						nextDateDay := preDate.AddDate(0, 0, 1)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TczSub(currentItem.Value, findItem.Value)

							if existVal, ok := existDataMap[av]; !ok {
								if _, existOk := existAddDataMap[av]; !existOk {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existAddDataMap[av] = av
							} else {
								if existVal != val {
									sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
									sql = fmt.Sprintf(sql, dataTableName)
									_, err = to.Raw(sql, val, edbInfoId, av).Exec()
									if err != nil {
										return err
									}
								}
							}
							break
						} else {
							preDateDay := preDate.AddDate(0, 0, -1)
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TczSub(currentItem.Value, findItem.Value)

								if existVal, ok := existDataMap[av]; !ok {
									if _, existOk := existAddDataMap[av]; !existOk {
										addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
										isAdd = true
									}
									existAddDataMap[av] = av
								} else {
									if existVal != val {
										sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
										sql = fmt.Sprintf(sql, dataTableName)
										_, err = to.Raw(sql, val, edbInfoId, av).Exec()
										if err != nil {
											return err
										}
									}
								}
								break
							}
						}
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}
