package request

// AddActivityReq 新增活动请求
type AddActivityReq struct {
	ActivityTypeId       int                 `description:"活动类型ID"`
	ActivityTypeName     string              `description:"活动类型名称"`
	ActivityName         string              `description:"活动标题"`
	StartTime            string              `description:"活动开始时间，格式：2021-11-09 18:08:44"`
	EndTime              string              `description:"活动结束时间，格式：2021-11-09 18:08:44"`
	Speaker              string              `description:"主讲人"`
	SpeakerHeadPic       string              `description:"主讲人头像"`
	SpeakerBackgroundPic string              `description:"主讲人背景图"`
	MainlandTel          string              `description:"大陆拨入"`
	HongKongTel          string              `description:"香港拨入"`
	TaiwanTel            string              `description:"台湾拨入"`
	AmericaTel           string              `description:"美国拨入"`
	SingaporeTel         string              `description:"新加坡拨入"`
	ParticipationCode    string              `description:"参会密码"`
	LinkParticipants     string              `description:"参会链接"`
	IsLimitPeople        int8                `description:"是否限制人数 1是，0否"`
	LimitPeopleNum       int                 `description:"限制人数数量"`
	ReportLink           string              `description:"报告链接"`
	ReportName           string              `description:"报告标题"`
	ReportId             int                 `description:"报告id"`
	City                 string              `description:"城市"`
	Address              string              `description:"活动地址"`
	Remarks              string              `description:"备注"`
	ArticleId            int                 `description:"报告链接所关联的文章ID"`
	PublishStatus        int8                `description:"是否发布，枚举值，不发布（保存）：0； 发布：1"`
	PosterList           []ActivityPosterReq `description:"海报文件列表"`
	PermissionIdList     []int               `description:"权限id列表"`
	IsNewReport          int                 `description:"是否关联新报告 0-否 1-是"`
}

// EditActivityReq 编辑活动请求
type EditActivityReq struct {
	ActivityId           int                 `description:"活动ID"`
	ActivityName         string              `description:"活动标题"`
	StartTime            string              `description:"活动开始时间，格式：2021-11-09 18:08:44"`
	EndTime              string              `description:"活动结束时间，格式：2021-11-09 18:08:44"`
	Speaker              string              `description:"主讲人"`
	SpeakerHeadPic       string              `description:"主讲人头像"`
	SpeakerBackgroundPic string              `description:"主讲人背景图"`
	MainlandTel          string              `description:"大陆拨入"`
	HongKongTel          string              `description:"香港拨入"`
	TaiwanTel            string              `description:"台湾拨入"`
	AmericaTel           string              `description:"美国拨入"`
	SingaporeTel         string              `description:"新加坡拨入"`
	ParticipationCode    string              `description:"参会密码"`
	LinkParticipants     string              `description:"参会链接"`
	IsLimitPeople        int8                `description:"是否限制人数 1是，0否"`
	LimitPeopleNum       int                 `description:"限制人数数量"`
	ReportLink           string              `description:"报告链接"`
	ReportName           string              `description:"报告标题"`
	ReportId             int                 `description:"报告id"`
	City                 string              `description:"城市"`
	Address              string              `description:"活动地址"`
	Remarks              string              `description:"备注"`
	ArticleId            int                 `description:"报告链接所关联的文章ID"`
	PublishStatus        int8                `description:"是否发布，枚举值，不发布（保存）：0； 发布：1"`
	VoiceList            []ActivityVoiceReq  `description:"音频文件列表"`
	PosterList           []ActivityPosterReq `description:"海报文件列表"`
	PermissionIdList     []int               `description:"权限id列表"`
	IsNewReport          int                 `description:"是否关联新报告 0-否 1-是"`
}

// ActivityVoiceReq 音频数据
type ActivityVoiceReq struct {
	Url         string `description:"音频资源url地址"`
	Name        string `description:"音频名称"`
	PlaySeconds string `description:"音频时长"`
}

// ActivityPosterReq 海报数据
type ActivityPosterReq struct {
	Url  string `description:"海报资源url地址"`
	Name string `description:"海报名称"`
}

// PublishActivityReq 发布活动请求
type PublishActivityReq struct {
	ActivityId int `description:"活动ID"`
}

// CreateQsReq 创建全时会议请求
type CreateQsReq struct {
	ActivityId int `description:"活动ID"`
	Time       int `description:"会议时长(分钟)"`
}

// CancelQsReq 取消全时会议请求
type CancelQsReq struct {
	ActivityId int `description:"活动ID"`
}

// QsShare2SellerReq 全时会议-分享给销售请求
type QsShare2SellerReq struct {
	QsId     int `description:"全时会议关联ID"`
	SellerId int `description:"销售ID"`
}

// ComeInShare2SellerReq 进门会议-分享给销售请求
type ComeInShare2SellerReq struct {
	ComeinEventId int `description:"进门会议ID"`
	SellerId      int `description:"销售ID"`
}
